﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Diagnostics;
using System.Reflection;
using System.Drawing;
using System.Drawing.Imaging;
using System.ComponentModel;

namespace Comparer
{
	class Exporters
	{
		private String _filename;
		private SourceGrid.Grid _tabela;
		private Exporter _exp;
		private SettingsClass _setc;
		private String _localcss;
		private String _outputcss;
		private StreamWriter tw;		
		private String _firstfile;
		private String _secondfile;
		
		public enum Exporter
		{
			CSV,
			HTML,
			IMAGE
		}

		public Exporters(String filename, SourceGrid.Grid tabela, SettingsClass setc )
		{
			this._filename = filename;
			this._tabela = tabela;			
			this._setc = setc;

			this._localcss = Assembly.GetExecutingAssembly().Location;
			_localcss = _localcss.Remove( _localcss.LastIndexOf( "\\" ) + 1 );
			_localcss = _localcss + "output.css";			
		}

		public void ExportCSV()
		{
			int ind = 0;
			String tmp = "";
			try
			{
				tw = new StreamWriter( this._filename, false, System.Text.Encoding.Default );

				//header

				if (this._setc.surquotes)
				{
					for (int i = 0; i < this._tabela.ColumnsCount; i++)
					{
						tmp = tmp + "\"" + this._tabela[0, i].Value.ToString() + "\"" + this._setc.sepchar;
					}

					tw.WriteLine( tmp );

					if (this._setc.showthumbs)
					{
						ind = 2;
					}
					else
					{
						ind = 1;
					}

					for (int k = ind; k < this._tabela.RowsCount; k++)
					{
						tmp = "";

						if (this._tabela[k, 0].ColumnSpan > 1)
						{
							tmp = "\"" + this._tabela[k, 0].Value.ToString() + "\"";

							for (int m = 1; m < this._tabela.ColumnsCount; m++)
							{
								tmp = tmp + this._setc.sepchar;
							}
						}
						else
						{
							for (int j = 0; j < this._tabela.ColumnsCount; j++)
							{
								tmp = tmp + "\"" + this._tabela[k, j].Value.ToString() + "\"" + this._setc.sepchar;
							}
						}

						tw.WriteLine( tmp );
					}
				}
				else
				{
					for (int i = 0; i < this._tabela.ColumnsCount; i++)
					{
						tmp = tmp + this._tabela[0, i].Value.ToString() + this._setc.sepchar;
					}

					tw.WriteLine( tmp );

					if (this._setc.showthumbs)
					{
						ind = 2;
					}
					else
					{
						ind = 1;
					}

					for (int k = ind; k < this._tabela.RowsCount; k++)
					{
						tmp = "";

						if (this._tabela[k, 0].ColumnSpan > 1)
						{
							tmp = this._tabela[k, 0].Value.ToString();

							for (int m = 1; m < this._tabela.ColumnsCount; m++)
							{
								tmp = tmp + this._setc.sepchar;
							}
						}
						else
						{
							for (int j = 0; j < this._tabela.ColumnsCount; j++)
							{
								tmp = tmp + this._tabela[k, j].Value.ToString() + this._setc.sepchar;
							}
						}

						tw.WriteLine( tmp );
					}
				}
				tw.Close();
			}
			catch (IOException ioe)
			{
				MessageBox.Show( this._setc.GetLangText( "ExportErrorPerm" ) );
			}
			catch (Exception exc)
			{
				MessageBox.Show( exc.ToString() );
			}
		}

		public void ExportHTML()
		{
			GenerateCSS();
			GenerateHTMLHead();
			GenerateHTMLContent();
			GenerateHTMLFooter();
		}

		private void GenerateCSS()
		{
			String tmphtml = this._filename;			
			tmphtml = tmphtml.Remove( 0, tmphtml.LastIndexOf( "\\" ) + 1 );
			tmphtml = tmphtml.Remove( tmphtml.LastIndexOf( "." ) );

			this._outputcss = this._filename;
			this._outputcss = this._outputcss.Remove( this._outputcss.LastIndexOf( "\\" ) + 1 );
			this._outputcss = this._outputcss + tmphtml + ".css";
			
			try
			{
				if (!File.Exists( this._localcss ))
				{
					//create CSS
					tw = new StreamWriter( this._outputcss );

					tw.WriteLine( "body\r\n{\r\nbackground-color: White;\r\nmargin: 0;\r\npadding: 0;text-align: center;\r\n}" );
					tw.WriteLine( "#main\r\n{\r\ntop: 0;\r\nright: 0;\r\nbottom: 0;\r\nleft: 0;\r\nwidth: 50%;\r\nmargin: auto;\r\n}" );
					tw.WriteLine( "h1\r\n{\r\nfont-weight:bold;\r\nfont-size:x-large;\r\ncolor: rgb(62,196,228);\r\n}" );
					tw.WriteLine( ".tablemain\r\n{\r\ntext-align:left;\r\nfont-size: small;\r\ncolor:Black;\r\nbackground:rgb(239,239,239);" );
					tw.WriteLine( "border: solid 1px rgb(62,196,228);\r\nempty-cells:show;\r\nborder-spacing:0px;\r\nborder-collapse:collapse;\r\n}" );
					tw.WriteLine( "th\r\n{\r\ntext-align:left;\r\nfont-weight:bold;\r\ncolor:Black;\r\nbackground: rgb(185,232,244);" );
					tw.WriteLine( "border: solid 1px rgb(62,196,228);\r\npadding: 2px 2px 2px 2px;\r\n}" );
					tw.WriteLine( "td\r\n{\r\nborder: solid 1px rgb(62, 196, 228);\r\npadding: 5px 5px 5px 5px;\r\n}" );
					tw.WriteLine( ".nofile\r\n{\r\nborder: solid 1px rgb(255,0,0);\r\npadding: 5px 5px 5px 5px;" );
					tw.WriteLine( "background:rgb(255,255,0);\r\ncolor:Black;\r\ntext-align:center;\r\n}" );
					tw.WriteLine( ".nofields\r\n{\r\nborder: solid 1px rgb(62, 196, 228);\r\npadding: 5px 5px 5px 5px;\r\n}" );
					tw.Close();
				}
				else
				{
					File.Copy( this._localcss, this._outputcss, true );
				}
			}
			catch (IOException ioe)
			{
				MessageBox.Show( this._setc.GetLangText( "ExportErrorPerm" ) );
			}
			catch (Exception exc)
			{
				MessageBox.Show( exc.ToString() );
			}
		}

		private void GenerateHTMLHead( )
		{
			try
			{
				tw = new StreamWriter( this._filename );

				tw.WriteLine( "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">" );
				tw.WriteLine( "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">" );
				tw.WriteLine( "<head>" );
				tw.WriteLine( "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />" );
				tw.WriteLine( "<meta name=\"keywords\" content=\"ComparerCmd,fenixproductions,\" />" );
				tw.WriteLine( "<meta name=\"description\" content=\"ComparerCmd - file's properties information.\" />" );
				tw.WriteLine( "<link rel=\"stylesheet\" type=\"text/css\" href=\"" + GetOutputCSSName() + "\" />" );
				tw.WriteLine( "<title>ComparerCmd</title>" );

				tw.WriteLine( "</head>\r\n<body>\r\n<div id=\"main\">" );				

				String tmphead = GetFromFile( "header.html" );

				if (tmphead.Equals( "" ))
				{
					tw.WriteLine( "<h1>" + GetFilesNames() + "</h1>" );
				}
				else
				{				
					tw.WriteLine( tmphead );
				}

				tw.WriteLine( "<table class=\"tablemain\" width=\"100%\">" );
			}
			catch (IOException ioe)
			{
				MessageBox.Show( this._setc.GetLangText( "ExportErrorPerm" ) );
			}
			catch (Exception exc)
			{
				MessageBox.Show( exc.ToString() );
			}
		}

		private void GenerateHTMLContent()
		{
			int ind = 0;
			String tmp = "";

			try
			{
				//header
				tmp = "<tr>";

				for (int i = 0; i < this._tabela.ColumnsCount; i++)
				{
					tmp = tmp + "<th>" + this._tabela[0, i].Value.ToString() + "</th>";
				}
				
				tmp = tmp + "</tr>";

				tw.WriteLine( tmp );

				ind = 1;

				if (this._setc.showthumbs)
				{

					String imgname = this._filename;
					imgname = imgname.Remove( 0, imgname.LastIndexOf( "\\" ) + 1 );
					imgname = imgname.Remove( imgname.LastIndexOf( '.' ) );
					String imgpath = this._filename;
					imgpath = imgpath.Remove( imgpath.LastIndexOf( "\\" ) + 1 );

					tmp = "<tr>";
					//generate images
					tmp = tmp + "<td>" + this._tabela[ind, 0].Value.ToString() + "</td>";

					for (int n = 1; n < this._tabela.ColumnsCount; n++)
					{
						tmp = tmp + "<td>";
						tmp = tmp + "<img class=\"thumb\" src=\"" + imgname + n.ToString() + ".png \"/ >";
						tmp = tmp + "</td>";
						Image img = (Bitmap) this._tabela[ind, n].Value;
						img.Save( imgpath + imgname + n.ToString() + ".png", ImageFormat.Png );
					}
					tmp = tmp + "</tr>";

					tw.WriteLine( tmp );
					ind = 2;					
				}

				for (int k = ind; k < this._tabela.RowsCount; k++)
				{
					tmp = "<tr>";

					if (this._tabela[k, 0].ColumnSpan > 1)
					{
						tmp = "<th colspan=\"" + this._tabela[k, 0].ColumnSpan.ToString() + "\">" + this._tabela[k, 0].Value.ToString() + "</th>";						
					}
					else
					{
						for (int j = 0; j < this._tabela.ColumnsCount; j++)
						{
							tmp = tmp + "<td>" + this._tabela[k, j].Value.ToString() + "</td>";
						}
					}

					tmp = tmp + "</tr>";
					tw.WriteLine( tmp );
				}
			}
			catch (IOException ioe)
			{
				MessageBox.Show( this._setc.GetLangText( "ExportErrorPerm" ) );
			}
			catch (Exception exc)
			{
				MessageBox.Show( exc.ToString() );
			}
		}

		private void GenerateHTMLFooter()
		{
			tw.WriteLine( "</table>\r\n</div>\r\n" );

			String tmpfoot = GetFromFile( "footer.html" );

			if (!tmpfoot.Equals( "" ))
			{				
				tw.WriteLine( tmpfoot );
			}

			tw.WriteLine( "</body>\r\n</html>" );
			tw.Close();
		}

		private String GetFromFile( String extfile )
		{
			String result = "";

			String myhead = Assembly.GetExecutingAssembly().Location;
			myhead = myhead.Remove( myhead.LastIndexOf( "\\" ) + 1 );
			myhead = myhead + extfile;

			if (File.Exists( myhead ))
			{
				TextReader tr = new StreamReader( myhead );

				String line = "";

				while ((line = tr.ReadLine()) != null)
				{
					result = result + line + "\r\n";
				}

				tr.Close();
			}

			return result;
		}

		private String GetOutputCSSName()
		{
			String outp = this._outputcss;
			outp = outp.Remove( 0, outp.LastIndexOf( "\\" ) + 1 );
			return outp;
		}

		private String GetFilesNames()
		{
			String tmp = "";
			if (this._setc.showthumbs)
			{
				switch (this._tabela.ColumnsCount)
				{
					case 2:
						tmp =  this._tabela[3, 1].Value.ToString();
						break;
					case 3:
						tmp = this._tabela[3, 1].Value.ToString();
						tmp = tmp + "&nbsp;|&nbsp;";
						tmp = tmp + this._tabela[3, 2].Value.ToString();
						break;
					default:
						tmp = "";
						break;
				}
			}
			else
			{
				switch (this._tabela.ColumnsCount)
				{
					case 1:
						tmp = this._tabela[2, 1].Value.ToString();
						break;
					case 2:
						tmp = this._tabela[2, 1].Value.ToString();
						tmp = tmp + "&nbsp;|&nbsp;";
						tmp = tmp + this._tabela[2, 2].Value.ToString();
						break;
					default:
						tmp = "";
						break;
				}
			}

			return tmp;
		}

		public void ExportImage( int format, String firstfile)
		{
			try
			{
				SourceGrid.Exporter.Image image = new SourceGrid.Exporter.Image();
				Bitmap bitmap = image.Export( this._tabela, this._tabela.CompleteRange );
				this._firstfile = firstfile;

				if (this._setc.imgshowhead)
				{
					Bitmap header = GenerateSingleHeader();
					Bitmap tmp;

					if (bitmap.Width >= header.Width)
					{
						tmp = new Bitmap( bitmap.Width, bitmap.Height + header.Height );						
					}
					else
					{
						//paste bitmap
						tmp = new Bitmap( header.Width, bitmap.Height + header.Height );						
					}
					
					Graphics gr = Graphics.FromImage( tmp );
					gr.FillRectangle( new SolidBrush( Color.White ), new Rectangle( 0, 0, tmp.Width, tmp.Height ) );
					gr.DrawImage( header, new Point( 0, 0 ) );
					gr.DrawImage( bitmap, new Point( 0, header.Height ) );
					bitmap = tmp;
				}
				

				switch (format)
				{
					case 1://bmp							
						bitmap.Save( this._filename, ImageFormat.Bmp );
						break;
					case 2://jpg							
						bitmap.Save( this._filename, ImageFormat.Jpeg );
						break;
					case 3://png							
						bitmap.Save( this._filename, ImageFormat.Png );
						break;
					default:
						break;
				}
			}
			catch (IOException ioe)
			{
				MessageBox.Show( this._setc.GetLangText( "ExportErrorPerm" ) );
			}
			catch (Exception exc)
			{
				MessageBox.Show( exc.ToString() );
			}
		}

		public void ExportImage( int format, String firstfile, String secondfile )
		{
			try
			{
				SourceGrid.Exporter.Image image = new SourceGrid.Exporter.Image();
				Bitmap bitmap = image.Export( this._tabela, this._tabela.CompleteRange );
				this._firstfile = firstfile;
				this._secondfile = secondfile;

				if (this._setc.imgshowhead)
				{
					Bitmap header = GenerateDoubleHeader();
					Bitmap tmp;

					if (bitmap.Width >= header.Width)
					{
						tmp = new Bitmap( bitmap.Width, bitmap.Height + header.Height );
					}
					else
					{
						//paste bitmap
						tmp = new Bitmap( header.Width, bitmap.Height + header.Height );
					}

					Graphics gr = Graphics.FromImage( tmp );
					gr.FillRectangle( new SolidBrush( Color.White ), new Rectangle( 0, 0, tmp.Width, tmp.Height ) );
					gr.DrawImage( header, new Point( 0, 0 ) );
					gr.DrawImage( bitmap, new Point( 0, header.Height ) );
					bitmap = tmp;
				}


				switch (format)
				{
					case 1://bmp							
						bitmap.Save( this._filename, ImageFormat.Bmp );
						break;
					case 2://jpg							
						bitmap.Save( this._filename, ImageFormat.Jpeg );
						break;
					case 3://png							
						bitmap.Save( this._filename, ImageFormat.Png );
						break;
					default:
						break;
				}
			}
			catch (IOException ioe)
			{
				MessageBox.Show( this._setc.GetLangText( "ExportErrorPerm" ) );
			}
			catch (Exception exc)
			{
				MessageBox.Show( exc.ToString() );
			}
		}

		private Bitmap GenerateSingleHeader()
		{
			Bitmap bmp = new Bitmap( this._setc.imgheadwidth, this._setc.imgheadheight );
			Graphics gr = Graphics.FromImage( bmp );
			SolidBrush br = new SolidBrush( this._setc.imgcolour );
			gr.FillRectangle( new SolidBrush( Color.White ), new Rectangle( 0, 0, bmp.Width, bmp.Height ) );

			String tmp = ImagesForm.CheckVariables( this._firstfile, this._setc.imgshead );

			TypeConverter converter = TypeDescriptor.GetConverter( typeof( FontStyle ) );
			FontStyle fs = (FontStyle) converter.ConvertFromString( this._setc.imgfontstyle );

			Font imgfont = new Font( new FontFamily( this._setc.imgfontname ), this._setc.imgfontsize, fs );
			
			gr.DrawString( tmp, imgfont, br, new Point( 0, 0 ) );
			return bmp;
		}

		private Bitmap GenerateDoubleHeader()
		{
			Bitmap bmp = new Bitmap( this._setc.imgheadwidth, this._setc.imgheadheight );
			Graphics gr = Graphics.FromImage( bmp );
			SolidBrush br = new SolidBrush( this._setc.imgcolour );
			gr.FillRectangle( new SolidBrush( Color.White ), new Rectangle( 0, 0, bmp.Width, bmp.Height ) );

			String tmp = ImagesForm.CheckVariables( this._firstfile, this._secondfile, this._setc.imgdhead );

			TypeConverter converter = TypeDescriptor.GetConverter( typeof( FontStyle ) );
			FontStyle fs = (FontStyle) converter.ConvertFromString( this._setc.imgfontstyle );

			Font imgfont = new Font( new FontFamily( this._setc.imgfontname ), this._setc.imgfontsize, fs );

			gr.DrawString( tmp, imgfont, br, new Point( 0, 0 ) );
			return bmp;
		}

	}
}
