﻿/* HISTORY:
 * 20081220 - added -c parameter support (works with files names or -L parameter)
 * 20081220 - customizable thumbnails height
 * 20081220 - selectable plugins fields
 * --------------------------------------------------------------------------------
 * 20081220 - fixed problem with folders and %L parameter
 * --------------------------------------------------------------------------------
 * 20081224 - main form refresh after plugins change
 * 20081224 - default plugins dir set
 * 20081224 - scan for plugins (in default dir)
 * 20081224 - little code claenup (%vars% to paths and vice versa)
 * --------------------------------------------------------------------------------
 * 20081225 - fix for autoscan (check if dir exists)
 * --------------------------------------------------------------------------------
 * 20081229 - fixed Explorer Thumbnail problem
 * 20081229 - replaced Tree with Grid
 * --------------------------------------------------------------------------------
 * 20081229 - lists with more than 2 elements were ignored
 * --------------------------------------------------------------------------------
 * 20081229 - Escape key handling added (Form2 and SettingsForm)
 * 20081229 - Ctrl+C - copy selected row as plain text
 * 20081229 - Shift+C - copy selected row as RTF text
 * 20081229 - icons added
 * 20081229 - HTML export added
 * 20081229 - BMP export added
 * 20081229 - stick mode added
 * --------------------------------------------------------------------------------
 * 20081230 - folders no longer can be dropped
 * 20081230 - stick for SINGLE extended (dropping on left will add on left, 
 *				right - right, column - replace) *				
 * --------------------------------------------------------------------------------
 * 20081231 - columns size is not changed on drop
 * --------------------------------------------------------------------------------
 * 20081231 - replaced menu with toolstrip (fix for stick mode - icons replace)
 * --------------------------------------------------------------------------------
 * 20090102 - removed Apply buttons (all tabs)!
 * 20090102 - fixed multiple comparers drop
 * 20090102 - added wordwrap option
 * 20090102 - add button (+) now shows OpenFile dialog,
 * --------------------------------------------------------------------------------
 * 20090103 - OK / Cancel buttons position corrected (Fields window)
 * 20090103 - Fields window's size remembered
 * --------------------------------------------------------------------------------
 * 20090105 - fixed ".." buttons in plugins Settings form,
 * 20090105 - fixed drag'n'drop related to ".." buttons,
 * 20090105 - CSV Export rewritten
 * 20090105 - HTML export rewritten (with images)
 * 20090105 - more formats for Images export
 * 20090105 - export error messages should be shown
 * 20090105 - ft_datetime error corrected
 * --------------------------------------------------------------------------------
 * 20090112 - added icon mode
 * 20090112 - fixed "show empty" option
 * --------------------------------------------------------------------------------
 * 20090112 - form is disabled if there is no plugins,
 * 20090112 - fixed extensions bug
 * --------------------------------------------------------------------------------
 * 20090112 - drag'ndrop did not activated panels
 * --------------------------------------------------------------------------------
 * 20090117 - Mercurial versionning system started (instead of CVS)
 * --------------------------------------------------------------------------------
 * 20090121 - corrected "show empty"
 * --------------------------------------------------------------------------------
 * 20090122 - added customizable output file names
 * --------------------------------------------------------------------------------
 * 20090122 - added customizable CSV separators and text quotes
 * --------------------------------------------------------------------------------
 * 20090126 - fixed few errors with "show empty" and "show different"
 * 20090126 - fixed colouring different items
 * 20090126 - files names always shown
 * --------------------------------------------------------------------------------
 * 20090126 - added image header generation
 * 20090127 - added font chooser
 * --------------------------------------------------------------------------------
 * 20090127 - added image header colour
 * 20090127 - added saving images with image header
 * 20090127 - added option for opening files with associated application
 * 20090127 - added date format description
 * --------------------------------------------------------------------------------
 * 20090225 - added wdx_proxy.dll handling (instead of internal solution; less problems with plugs)
 * 20090225 - removed columns sorting
 * --------------------------------------------------------------------------------
 * 20090226 - optimized wdx_proxy usage (InitializePlugins added to PluginProxy class)
 * 20090226 - fixed showempty and show different relation
 * --------------------------------------------------------------------------------
 * 20090301 - added language files handling
 --------------------------------------------------------------------------------
 * 20090303 - added configurations combobox
 --------------------------------------------------------------------------------
 * 20090304 - added shortcuts handling (Alt+Enter, Alt+Delete)
 * 20090304 - translation corrections
 --------------------------------------------------------------------------------
 * 20090305 - few SettingsForm GUI changes according to French translation,
 * 20090305 - small lng strings corrections  ( ... replaced with … ), added ByteSuffix
 --------------------------------------------------------------------------------
 * 20090306 - licence string added, changed "Application's window" string to "Main window" 
 --------------------------------------------------------------------------------
 * 20090504 - use default values if there is no INI
 * 20090504 - create "configs" directory if does not exist (it was giving errors!)
 * 20090504 - "default" INI name language dependant
 --------------------------------------------------------------------------------
 * 20090506 - "Scan for plugins" generated errors for new config (reason -> not enabled controls)
 --------------------------------------------------------------------------------
 * 20090509 - ThreadAbortException handled
 * 20090509 - added skipping FileDesc.wdx (hardcoded name in Scan for plugins)
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace Comparer
{	
	public delegate void RefreshFormEventHandler( object sender );

	public partial class Form2 : Form
	{
		public String firstFile = "";
		public String secondFile = "";
		private SettingsClass setc;
		public StartMode mode = StartMode.NONE;
		public String[] startparams;		
		public String inifile = "";
		private ThumbStarter ts;
		private static BackgroundWorker bw;
		private List<String> lista;
		private List<String> lista2;
		private List<SettElem> _custominis;
		private string _startini;

		public enum StartMode
		{
			NONE = 0,
			SINGLE,
			DOUBLE,
			LIST
		}

		public enum eColumns
		{
			// this must match the order at which the columns are added
			prop = 0,
			first = 1,
			second = 2,
		}

		public Form2()
		{
			InitializeComponent();

			this.inifile = Application.ExecutablePath;
			this.inifile = inifile.Remove( inifile.LastIndexOf( "\\" ) + 1 );
			this.inifile = inifile + "comparer.ini";

			startparams = Environment.GetCommandLineArgs();

			switch (startparams.Length)
			{
				case 1:
					this.mode = StartMode.NONE;
					break;
				case 2:
					this.mode = StartMode.SINGLE;
					firstFile = startparams[1];

					if (!File.Exists( firstFile ))
					{
						this.mode = StartMode.NONE;
					}
					break;
				case 3:
					if (startparams[1].ToUpper().Equals( "-L" ))
					{
						this.mode = StartMode.LIST;
						firstFile = startparams[2];

						if (!File.Exists( firstFile ))
						{
							this.mode = StartMode.NONE;
						}
					}
					else
					{
						if (startparams[1].ToUpper().Equals( "-C" ))
						{
							this.mode = StartMode.NONE;
							this.inifile = startparams[2].ToString();
						}
						else
						{
							this.mode = StartMode.NONE;
							firstFile = startparams[1];
							secondFile = startparams[2];

							if (File.Exists( firstFile ) && File.Exists( secondFile ))
							{
								this.mode = StartMode.DOUBLE;
							}
							else
							{
								if (!File.Exists( firstFile ))
								{
									if (File.Exists( secondFile ))
									{
										this.mode = StartMode.SINGLE;
										firstFile = secondFile;
										secondFile = "";
									}
									else
									{
										this.mode = StartMode.NONE;
										firstFile = "";
										secondFile = "";
									}
								}
							}
						}

					}
					break;
				case 4:
					if (startparams[1].ToUpper().Equals( "-C" ))
					{
						this.inifile = startparams[2].ToString();

						firstFile = startparams[3];

						if (!File.Exists( firstFile ))
						{
							this.mode = StartMode.NONE;
						}
						else
						{
							this.mode = StartMode.SINGLE;
						}
					}
					else
					{
						this.inifile = startparams[2].ToString();
						this.mode = StartMode.NONE;
					}
					break;
				case 5:
					if (startparams[1].ToUpper().Equals( "-C" ))
					{
						this.inifile = startparams[2].ToString();

						if (startparams[3].ToUpper().Equals( "-L" ))
						{
							this.mode = StartMode.LIST;
							firstFile = startparams[4];

							if (!File.Exists( firstFile ))
							{
								this.mode = StartMode.NONE;
							}							
						}
						else
						{
							firstFile = startparams[3];
							secondFile = startparams[4];

							if (File.Exists( firstFile ) && File.Exists( secondFile ))
							{
								this.mode = StartMode.DOUBLE;
							}
							else
							{
								if (!File.Exists( firstFile ))
								{
									if (File.Exists( secondFile ))
									{
										this.mode = StartMode.SINGLE;
										firstFile = secondFile;
										secondFile = "";
									}
									else
									{
										this.mode = StartMode.NONE;
										firstFile = "";
										secondFile = "";
									}
								}
							}
						}
					}
					else
					{
						this.inifile = startparams[2].ToString();
						this.mode = StartMode.NONE;
					}
					break;
				default:
					this.mode = StartMode.NONE;
					break;
			}			
		}

		private void Form2_Load( object sender, EventArgs e )
		{
			setc = new SettingsClass( this.inifile );
			//MessageBox.Show( this.inifile );
			setc.ReadGeneral();			
			SetFormLang();

			this.Width = setc.sizeX;
			this.Height = setc.sizeY;
			this.TopMost = setc.alwaystop;		
	
			//TODO: custom INIs handling
			this._startini = this.inifile;

			String tmpini = "%APP_PATH%\\";
			tmpini = setc.AppPathToExe( tmpini );
			tmpini = tmpini + "configs\\" + this.setc.lastinifile;


			if (this.setc.rememberlastini)
			{
				if (File.Exists( tmpini ))
				{
					this.inifile = tmpini;
				}
			}

			/*
			if (CheckCustomInis())
			{
				FillInisCombo();
			}
			 * */

			CheckCustomInis();
			FillInisCombo();


			//modes
			switch (mode)
			{
				case StartMode.NONE:
					GenerateTable();
					compareTSDB.Enabled = false;
					exportTSDB.Enabled = false;
					stickmodeTSB.Enabled = true;
					stickmodeTSB.CheckState = BooleanToCheckState(setc.stickmode);
					ValidateStickCheck();
					break;
				case StartMode.SINGLE:
					GetSingleFileInfo( this.firstFile );
					compareTSDB.Enabled = false;
					exportTSDB.Enabled = true;
					stickmodeTSB.Enabled = true;
					stickmodeTSB.CheckState = BooleanToCheckState( setc.stickmode );
					ValidateStickCheck();
					break;
				case StartMode.DOUBLE:
					GetDoubleFilesInfo( firstFile, secondFile );
					GenerateCustomCompareMenu();
					exportTSDB.Enabled = true;
					stickmodeTSB.Enabled = true;
					stickmodeTSB.CheckState = BooleanToCheckState( setc.stickmode );
					ValidateStickCheck();
					break;
				case StartMode.LIST:
					//read listed files						
					StreamReader sr = new StreamReader( firstFile, Encoding.Default );

					List<String> files = new List<String>();
					String input = "";

					while ((input = sr.ReadLine()) != null)
					{
						if (File.Exists( input ))
						{
							files.Add( input );
						}
					}

					sr.Close();

					if (files.Count > 2)
					{
						String tmp1 = files[0].ToString();
						String tmp2 = files[1].ToString();
						files = new List<String>();
						files.Add( tmp1 );
						files.Add( tmp2 );
					}

					switch (files.Count)
					{
						case 1:
							this.mode = StartMode.SINGLE;
							this.firstFile = files[0].ToString();
							GetSingleFileInfo( firstFile );
							compareTSDB.Enabled = false;
							exportTSDB.Enabled = true;
							stickmodeTSB.Enabled = true;
							stickmodeTSB.CheckState = BooleanToCheckState( setc.stickmode );
							ValidateStickCheck();
							break;
						case 2:
							this.mode = StartMode.DOUBLE;
							firstFile = files[0].ToString();
							secondFile = files[1].ToString();
							GetDoubleFilesInfo( firstFile, secondFile );
							GenerateCustomCompareMenu();
							exportTSDB.Enabled = true;
							stickmodeTSB.Enabled = true;
							stickmodeTSB.CheckState = BooleanToCheckState( setc.stickmode );
							ValidateStickCheck();
							break;
						case 0:
							this.mode = StartMode.NONE;
							compareTSDB.Enabled = false;
							exportTSDB.Enabled = false;
							stickmodeTSB.Enabled = false;
							stickmodeTSB.CheckState = BooleanToCheckState( setc.stickmode );
							ValidateStickCheck();
							break;
						default:
							break;
					}

					break;
				default:
					break;
			}
			detailsGrid.Focus();						
		}

		private void GenerateTable()
		{			
			detailsGrid.BorderStyle = BorderStyle.FixedSingle;
			detailsGrid.Dock = DockStyle.Fill;

			switch (mode)
			{
				//case StartMode.NONE:			
				case StartMode.SINGLE:
					GenerateSingleColumns();					
					break;
				case StartMode.NONE:
				case StartMode.DOUBLE:
					GenerateDoubleColumns();					
					break;				
				case StartMode.LIST:					
				default:
					break;
			}				
		}

		private void GetSingleFileInfo( String filename )
		{
			if (ts != null)
			{
				ts.CleanThreads();
			}

			GenerateSingleColumns();
			CreateGeneralInfo( filename );			

			String ext = filename;
			ext = ext.Remove( 0, ext.LastIndexOf( "." ) );

			setc.ReadPlugins( ext );

			CreatePluginsInfo( filename );

			if (setc.showthumbs)
			{
				//thumbnails

				setc.ReadWLXPlugins( ext );

				lista = new List<string>();
				for (int i = 0; i < setc.wlxplugins.Count; i++)
				{
					lista.Add( setc.wlxplugins[i]._path );
				}

				bw = new BackgroundWorker();
				bw.DoWork += new DoWorkEventHandler( bw_DoWork );
				bw.RunWorkerCompleted += new RunWorkerCompletedEventHandler( bw_RunWorkerCompleted );
				bw.RunWorkerAsync();				
			}

			CheckWrapping();
		}

		void bw_RunWorkerCompleted( object sender, RunWorkerCompletedEventArgs e )
		{
			switch (mode)
			{				
				case StartMode.SINGLE:
					if (ts.pict1 != null)
					{
						if (!ts.icon1)
						{
							detailsGrid[1, 1].Value = ts.pict1;
						}
						else
						{
							if (setc.showicons)
							{
								detailsGrid.Rows[1].Height = 32;
								detailsGrid[1, 1].Value = ts.pict1;
							}
							else
							{
								if (!setc.showempty)
								{
									detailsGrid.Rows[1].Visible = false;
								}
							}
						}
					}
					else
					{
						if (!setc.showempty)
						{
							detailsGrid.Rows[1].Visible = false;
						}
					}
					break;
				case StartMode.DOUBLE:
					if ((ts.pict1 == null) && (ts.pict2 == null))
					{
						if (setc.showicons)
						{
							detailsGrid.Rows[1].Height = 32;
							detailsGrid.Rows[1].Visible = false;
						}
						else
						{
							if (!setc.showempty)
							{
								detailsGrid.Rows[1].Visible = false;
							}
						}
					}
					else
					{					
						if (ts.icon1 && ts.icon2)
						{
							if (!setc.showicons && !setc.showempty)
							{
								detailsGrid.Rows[1].Visible = false;
							}
							else
							{
								if (setc.showicons)
								{
									detailsGrid.Rows[1].Height = 32;

									if (ts.pict1 != null)
									{
										detailsGrid[1, 1].Value = ts.pict1;
									}

									if (ts.pict2 != null)
									{
										detailsGrid[1, 2].Value = ts.pict2;
									}
								}
							}
						}
						else
						{
							if (ts.pict1 != null)
							{
								detailsGrid[1, 1].Value = ts.pict1;
							}
							if (ts.pict2 != null)
							{
								detailsGrid[1, 2].Value = ts.pict2;
							}
						}					
					}
					break;
				default:
					break;
			}

			//throw new NotImplementedException();
		}

		void bw_DoWork( object sender, DoWorkEventArgs e )
		{
			switch (mode)
			{
				case StartMode.SINGLE:
					ts = new ThumbStarter();
					ts.StartProcedure( firstFile, detailsGrid.Columns[1].Width, setc.thumbsheight, lista );
					break;
				case StartMode.DOUBLE:
					ts = new ThumbStarter();
					ts.StartProcedures( firstFile, secondFile, detailsGrid.Columns[1].Width, detailsGrid.Columns[2].Width, setc.thumbsheight, lista, lista2 );
					break;	
				default:
					break;
			}
			
			//throw new NotImplementedException();
		}

		private void GetDoubleFilesInfo( String filename1, String filename2 )
		{
			if (ts != null)
			{
				ts.CleanThreads();
			}

			GenerateDoubleColumns();
			CreateGeneralInfo( filename1, filename2 );
			
			String ext1 = filename1;
			ext1 = ext1.Remove( 0, ext1.LastIndexOf( "." ) );

			String ext2 = filename2;
			ext2 = ext2.Remove( 0, ext2.LastIndexOf( "." ) );
			setc.plugins = new List<Plug>();
			setc.ReadPlugins( ext1, ext2 );

			CreatePluginsInfo( filename1, filename2 );

			if (setc.showthumbs)
			{

				setc.ReadWLXPlugins( ext1 );

				lista = new List<string>();
				
				for (int i = 0; i < setc.wlxplugins.Count; i++)
				{
					lista.Add( setc.wlxplugins[i]._path );
				}
				
				setc.ReadWLXPlugins( ext1 );
				
				lista2 = new List<string>();
				for (int j = 0; j < setc.wlxplugins.Count; j++)
				{
					lista2.Add( setc.wlxplugins[j]._path );
				}

				bw = new BackgroundWorker();
				bw.DoWork += new DoWorkEventHandler( bw_DoWork );
				bw.RunWorkerCompleted += new RunWorkerCompletedEventHandler( bw_RunWorkerCompleted );
				bw.RunWorkerAsync();				
			}

			CheckWrapping();
		}

		private void GenerateCustomCompareMenu()
		{
			compareTSDB.DropDownItems.Clear();

			String ext1 = this.firstFile;
			ext1 = ext1.Remove( 0, ext1.LastIndexOf( "." ) );

			String ext2 = this.secondFile;
			ext2 = ext2.Remove( 0, ext2.LastIndexOf( "." ) );

			this.setc.ReadComparers( ext1, ext2 );

			if (setc.comparers.Count > 0)
			{
				//MessageBox.Show( setc.comparers.Count.ToString() );
				compareTSDB.Enabled = true;

				for (int i = 0; i < setc.comparers.Count; i++)
				{
					if (!setc.comparers[i]._path.Equals( "" ))
					{
						ToolStripMenuItem tsmi = new ToolStripMenuItem();
						tsmi.Name = "comparer" + i.ToString() + "TSMI";
						tsmi.Size = new System.Drawing.Size( 152, 22 );
						tsmi.Text = setc.comparers[i]._name;
						tsmi.Tag = (Comparer) setc.comparers[i];

						tsmi.Click += new System.EventHandler( this.CustomCompareMenu_Click );
						compareTSDB.DropDownItems.Add( tsmi );
					}
				}
			}
		}
				
		private void GenerateSingleColumns()
		{
			detailsGrid.Rows.Clear();
			detailsGrid.BorderStyle = BorderStyle.FixedSingle;
			detailsGrid.Dock = DockStyle.Fill;
			detailsGrid.SpecialKeys = SourceGrid.GridSpecialKeys.Default ^ SourceGrid.GridSpecialKeys.Escape;
			detailsGrid.KeyPress += new KeyPressEventHandler( General_KeyPress );

			detailsGrid.ColumnsCount = 2;
			detailsGrid.FixedRows = 1;
			int ind = detailsGrid.RowsCount;
			detailsGrid.Rows.Insert( ind );
			SourceGrid.Cells.ColumnHeader kk = new SourceGrid.Cells.ColumnHeader( setc.GetLangText( "HeadProperty" ) );
			kk.SortComparer = null;
			kk.AutomaticSortEnabled = false;
			
			detailsGrid[ind, 0] = kk;
			kk = new SourceGrid.Cells.ColumnHeader( setc.GetLangText( "HeadFile" ) );
			kk.SortComparer = null;
			kk.AutomaticSortEnabled = false;			
			detailsGrid[ind, 1] = kk;
			ind++;

			if (setc.remembersize)
			{
				detailsGrid.Columns[0].Width = setc.size_label;
				detailsGrid.Columns[1].Width = setc.size_scol;
			}
			else
			{
				detailsGrid.Columns[0].Width = 150;
				detailsGrid.Columns[1].Width = this.detailsGrid.Width - detailsGrid.Columns[0].Width - 30;
			}

			if (setc.showthumbs)
			{
				detailsGrid.Rows.Insert( 1 );
				detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowThumbnail" ) );
				detailsGrid[ind, 0].Editor = null;

				detailsGrid[ind, 1] = new SourceGrid.Cells.Image( global::Comparer.Properties.Resources.nopreview );		
				detailsGrid.Rows[1].Height = setc.thumbsheight;
				detailsGrid[ind, 1].Editor.EnableEdit = false;
			}		

			detailsGrid.Columns.ColumnWidthChanged += new SourceGrid.ColumnInfoEventHandler(Columns_ColumnWidthChanged);
		}

		private void GenerateDoubleColumns()
		{
			detailsGrid.Rows.Clear();
			detailsGrid.BorderStyle = BorderStyle.FixedSingle;
			detailsGrid.Dock = DockStyle.Fill;
			detailsGrid.SpecialKeys = SourceGrid.GridSpecialKeys.Default ^ SourceGrid.GridSpecialKeys.Escape;
			detailsGrid.KeyPress += new KeyPressEventHandler( General_KeyPress );

			detailsGrid.ColumnsCount = 3;
			detailsGrid.FixedRows = 1;
			int ind = detailsGrid.RowsCount;
			detailsGrid.Rows.Insert( ind );

			SourceGrid.Cells.ColumnHeader kk = new SourceGrid.Cells.ColumnHeader( setc.GetLangText( "HeadProperty" ) );
			kk.SortComparer = null;
			kk.AutomaticSortEnabled = false;
			//detailsGrid[ind, 0] = new SourceGrid.Cells.ColumnHeader( "Property" );
			detailsGrid[ind, 0] = kk;
			detailsGrid.Columns[0].Width = 150;

			kk = new SourceGrid.Cells.ColumnHeader( setc.GetLangText( "HeadFirstFile" ) );
			kk.SortComparer = null;
			kk.AutomaticSortEnabled = false;
			//detailsGrid[ind, 1] = new SourceGrid.Cells.ColumnHeader( "First file" );
			detailsGrid[ind, 1] = kk;

			kk = new SourceGrid.Cells.ColumnHeader( setc.GetLangText( "HeadSecondFile" ) );
			kk.SortComparer = null;
			kk.AutomaticSortEnabled = false;
			//detailsGrid[ind, 2] = new SourceGrid.Cells.ColumnHeader( "Second file" );
			detailsGrid[ind, 2] = kk;
			ind++;
			if (setc.remembersize)
			{
				detailsGrid.Columns[0].Width = setc.size_label;
				detailsGrid.Columns[1].Width = setc.size_dcol1;
				detailsGrid.Columns[2].Width = setc.size_dcol2;
			}
			else
			{
				detailsGrid.Columns[0].Width = 150;
				detailsGrid.Columns[1].Width = (this.detailsGrid.Width - detailsGrid.Columns[0].Width - 30) / 2;
				detailsGrid.Columns[2].Width = (this.detailsGrid.Width - detailsGrid.Columns[0].Width - 30) / 2;
			}

			if (setc.showthumbs)
			{
				detailsGrid.Rows.Insert( ind );
				detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowThumbnail" ) );
				detailsGrid[ind, 0].Editor = null;

				detailsGrid[ind, 1] = new SourceGrid.Cells.Image( global::Comparer.Properties.Resources.nopreview );
				detailsGrid.Rows[ind].Height = setc.thumbsheight;
				detailsGrid[ind, 1].Editor.EnableEdit = false;

				detailsGrid[ind, 2] = new SourceGrid.Cells.Image( global::Comparer.Properties.Resources.nopreview );
				detailsGrid.Rows[ind].Height = setc.thumbsheight;
				detailsGrid[ind, 2].Editor.EnableEdit = false;				
			}

			detailsGrid.Columns.ColumnWidthChanged += new SourceGrid.ColumnInfoEventHandler( Columns_ColumnWidthChanged );
		}

		void Columns_ColumnWidthChanged( object sender, SourceGrid.ColumnInfoEventArgs e )
		{
			//throw new NotImplementedException();

			switch (mode)
			{				
				case StartMode.SINGLE:
					switch (e.Column.Index)
					{
						case 0:
							setc.size_label = e.Column.Width;
							break;
						case 1:
							setc.size_scol = e.Column.Width;
							break;					
						default:
							break;
					}					
					break;
				case StartMode.DOUBLE:
					switch (e.Column.Index)
					{
						case 0:
							setc.size_label = e.Column.Width;
							break;
						case 1:
							setc.size_dcol1 = e.Column.Width;
							break;
						case 2:
							setc.size_dcol2 = e.Column.Width;
							break;
						default:
							break;
					}					
					break;				
				default:
					break;
			}

			CheckWrapping(e.Column.Index);
		}

		private void CreateGeneralInfo( String firstFile )
		{
			int ind = detailsGrid.RowsCount;
			detailsGrid.Rows.Insert( ind );

			SourceGrid.Cells.Views.Cell selView = new SourceGrid.Cells.Views.Cell();
			selView.Font = new Font( this.Font.FontFamily, this.Font.Size, FontStyle.Bold );
						
			detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowGeneral" ) );
			detailsGrid[ind, 0].ColumnSpan = 2;
			detailsGrid[ind, 0].View = selView;			
			ind++;

			FileInfo file1 = new FileInfo( firstFile );

			//Name
			detailsGrid.Rows.Insert( ind );
			detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowName" ) );
			detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( file1.Name.ToString() );
			ind++;
						
			//Path
			detailsGrid.Rows.Insert( ind );
			detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowPath" ) );
			detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( file1.DirectoryName.ToString() );
			ind++;
			
			//Size
			detailsGrid.Rows.Insert( ind );
			detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowSize" ) );
			detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( file1.Length.ToString() + " " + setc.GetLangText( "ByteSuffix" ) );
			ind++;
			
			// Attributes
			detailsGrid.Rows.Insert( ind );
			detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowAttributes" ) );
			detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( file1.Attributes.ToString() );
			ind++;
			
			// Created
			detailsGrid.Rows.Insert( ind );
			detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowCreated" ) );
			detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( file1.CreationTime.ToString() );
			ind++;

			// Last access
			detailsGrid.Rows.Insert( ind );
			detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowAccess" ) );
			detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( file1.LastAccessTime.ToString() );
			ind++;

			// Last write
			detailsGrid.Rows.Insert( ind );
			detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowWrite" ) );
			detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( file1.LastWriteTime.ToString() );

		}

		private void CreateGeneralInfo( String firstFile, String secondFile )
		{
			int ind = detailsGrid.RowsCount;
			detailsGrid.Rows.Insert( ind );			

			SourceGrid.Cells.Views.Cell selView = new SourceGrid.Cells.Views.Cell();
			selView.Font = new Font( this.Font.FontFamily, this.Font.Size, FontStyle.Bold );

			detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowGeneral" ) );
			detailsGrid[ind, 0].ColumnSpan = 3;
			detailsGrid[ind, 0].View = selView;
			ind++;

			FileInfo file1 = new FileInfo( firstFile );
			FileInfo file2 = new FileInfo( secondFile );
			String tmp1 = "";
			String tmp2 = "";
			

			//Name
			tmp1 = file1.Name.ToString();
			tmp2 = file2.Name.ToString();

			detailsGrid.Rows.Insert( ind );
			detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowName" ) );
			detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( tmp1 );
			detailsGrid[ind, 2] = new SourceGrid.Cells.Cell( tmp2 );

			if (!tmp1.ToUpper().Equals( tmp2.ToUpper() ) && !(this.setc.showdifferent))
			{
				SourceGrid.Cells.Views.Cell selView2 = new SourceGrid.Cells.Views.Cell();
				selView2.ForeColor = Color.Red;
				detailsGrid[ind, 1].View = selView2;
				detailsGrid[ind, 2].View = selView2;
			}
			ind++;			
						
			//Path
			tmp1 = file1.DirectoryName.ToString();
			tmp2 = file2.DirectoryName.ToString();
			
			if (tmp1.Equals( tmp2 ) && (this.setc.showdifferent))
			{
				//
			}
			else
			{
				detailsGrid.Rows.Insert( ind );
				detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowPath" ) );
				detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( tmp1 );				
				detailsGrid[ind, 2] = new SourceGrid.Cells.Cell( tmp2 );

				if (!tmp1.Equals( tmp2 ) && !(this.setc.showdifferent))
				{
					SourceGrid.Cells.Views.Cell selView2 = new SourceGrid.Cells.Views.Cell();
					selView2.ForeColor = Color.Red;					
					detailsGrid[ind, 1].View = selView2;
					detailsGrid[ind, 2].View = selView2;
				}

				ind++;
			}

			//Size
			tmp1 = file1.Length.ToString() + " " + setc.GetLangText("ByteSuffix");
			tmp2 = file2.Length.ToString() + " " + setc.GetLangText( "ByteSuffix" );
			if (tmp1.Equals( tmp2 ) && (this.setc.showdifferent))
			{
				//
			}
			else
			{
				detailsGrid.Rows.Insert( ind );
				detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowSize" ) );
				detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( tmp1 );
				detailsGrid[ind, 2] = new SourceGrid.Cells.Cell( tmp2 );

				if (!tmp1.Equals( tmp2 ) && !(this.setc.showdifferent))
				{
					SourceGrid.Cells.Views.Cell selView2 = new SourceGrid.Cells.Views.Cell();
					selView2.ForeColor = Color.Red;					
					detailsGrid[ind, 1].View = selView2;
					detailsGrid[ind, 2].View = selView2;
				}

				ind++;
			}

			// Attributes
			tmp1 = file1.Attributes.ToString();
			tmp2 = file2.Attributes.ToString();
			if (tmp1.Equals( tmp2 ) && (this.setc.showdifferent))
			{
				//
			}
			else
			{
				detailsGrid.Rows.Insert( ind );
				detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowAttributes" ) );
				detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( tmp1 );
				detailsGrid[ind, 2] = new SourceGrid.Cells.Cell( tmp2 );

				if (!tmp1.Equals( tmp2 ) && !(this.setc.showdifferent))
				{
					SourceGrid.Cells.Views.Cell selView2 = new SourceGrid.Cells.Views.Cell();
					selView2.ForeColor = Color.Red;					
					detailsGrid[ind, 1].View = selView2;
					detailsGrid[ind, 2].View = selView2;
				}
				
				ind++;
			}

			// Created
			tmp1 = file1.CreationTime.ToString();
			tmp2 = file2.CreationTime.ToString();
			if (tmp1.Equals( tmp2 ) && (this.setc.showdifferent))
			{
				//
			}
			else
			{
				detailsGrid.Rows.Insert( ind );
				detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowCreated" ) );
				detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( tmp1 );
				detailsGrid[ind, 2] = new SourceGrid.Cells.Cell( tmp2 );

				if (!tmp1.Equals( tmp2 ) && !(this.setc.showdifferent))
				{
					SourceGrid.Cells.Views.Cell selView2 = new SourceGrid.Cells.Views.Cell();
					selView2.ForeColor = Color.Red;					
					detailsGrid[ind, 1].View = selView2;
					detailsGrid[ind, 2].View = selView2;
				}
				
				ind++;
			}

			// Last access
			tmp1 = file1.LastAccessTime.ToString();
			tmp2 = file2.LastAccessTime.ToString();
			if (tmp1.Equals( tmp2 ) && (this.setc.showdifferent))
			{
				//
			}
			else
			{
				detailsGrid.Rows.Insert( ind );
				detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowAccess" ) );
				detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( tmp1 );
				detailsGrid[ind, 2] = new SourceGrid.Cells.Cell( tmp2 );

				if (!tmp1.Equals( tmp2 ) && !(this.setc.showdifferent))
				{
					SourceGrid.Cells.Views.Cell selView2 = new SourceGrid.Cells.Views.Cell();
					selView2.ForeColor = Color.Red;					
					detailsGrid[ind, 1].View = selView2;
					detailsGrid[ind, 2].View = selView2;
				}
				
				ind++;
			}

			// Last write
			tmp1 = file1.LastWriteTime.ToString();
			tmp2 = file2.LastWriteTime.ToString();
			if (tmp1.Equals( tmp2 ) && (this.setc.showdifferent))
			{
				//
			}
			else
			{
				detailsGrid.Rows.Insert( ind );
				detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "RowWrite" ) );
				detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( tmp1 );
				detailsGrid[ind, 2] = new SourceGrid.Cells.Cell( tmp2 );

				if (!tmp1.Equals( tmp2 ) && !(this.setc.showdifferent))
				{
					SourceGrid.Cells.Views.Cell selView2 = new SourceGrid.Cells.Views.Cell();
					selView2.ForeColor = Color.Red;					
					detailsGrid[ind, 1].View = selView2;
					detailsGrid[ind, 2].View = selView2;
				}
				
				ind++;
			}
		}

		private void CreatePluginsInfo( String firstFile )
		{			
			int ind = detailsGrid.RowsCount;

			if (setc.plugins.Count > 0)
			{
				PluginProxy plug = new PluginProxy();
				plug.Load();

				String tmpplugs = "";
				for (int i = 0; i < setc.plugins.Count; i++)
				{
					tmpplugs = tmpplugs + setc.plugins[i]._path + "?";
				}

				tmpplugs = tmpplugs.TrimEnd( '?' );
				plug.InitializePlugins( tmpplugs );

				for (int i = 0; i < setc.plugins.Count; i++)
				{
					List<ValueElem> valist = new List<ValueElem>();
					String tmpfieldstr = plug.GetFields( i );
					String[] tmpfields = tmpfieldstr.Split( ';' );

					String tmp1 = "";
					String tmpflds = "";

					for (int j = 0; j < tmpfields.Length; j++)
					{
						tmpflds = setc.plugins[i]._fields;

						if (tmpflds.Contains( "*" ) || tmpflds.Contains( ";" + j.ToString() + ";" ))
						{
							Debug.WriteLine( "name:\t" + tmpfields[j] );
							//tmp1 = plug.GetValue( j, firstFile, setc.plugins[i]._path );
							tmp1 = plug.GetValueStr( tmpfields[j], firstFile, i );
							tmp1 = tmp1.TrimEnd( Convert.ToChar( (Int32) 0 ) );

							if (!tmp1.Equals( "" ) || setc.showempty)
							{
								valist.Add( new ValueElem( tmpfields[j], tmp1 ) );
							}
						}
					}					

					if (valist.Count > 0)
					{
						//int fieldsnr = 0;

						SourceGrid.Cells.Views.Cell selView = new SourceGrid.Cells.Views.Cell();
						selView.Font = new Font( this.Font.FontFamily, this.Font.Size, FontStyle.Bold );

						detailsGrid.Rows.Insert( ind );
						detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.plugins[i]._name );
						detailsGrid[ind, 0].ColumnSpan = 2;
						detailsGrid[ind, 0].View = selView;
						ind++;

						for (int k = 0; k < valist.Count; k++)
						{
							detailsGrid.Rows.Insert( ind );
							detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( valist[k].Field );
							detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( valist[k].Value );
							ind++;
						}
					}
					else
					{
						if (setc.showempty)
						{
							SourceGrid.Cells.Views.Cell selView = new SourceGrid.Cells.Views.Cell();
							selView.Font = new Font( this.Font.FontFamily, this.Font.Size, FontStyle.Bold );

							detailsGrid.Rows.Insert( ind );
							detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.plugins[i]._name + setc.GetLangText( "EmptyError" ) );
							detailsGrid[ind, 0].ColumnSpan = 2;
							detailsGrid[ind, 0].View = selView;
						}
					}

				}
				plug.Free();

			}
			else
			{
				SourceGrid.Cells.Views.Cell selView = new SourceGrid.Cells.Views.Cell();
				selView.Font = new Font( this.Font.FontFamily, this.Font.Size, FontStyle.Bold );

				detailsGrid.Rows.Insert( ind );
				detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "NoPlugins" ) );
				detailsGrid[ind, 0].ColumnSpan = 2;
				detailsGrid[ind, 0].View = selView;
			}
		}

		private void CreatePluginsInfo( String firstFile, String secondFile )
		{
			int ind = detailsGrid.RowsCount;

			if (setc.plugins.Count > 0)
			{
				PluginProxy plug = new PluginProxy();
				plug.Load();

				String tmpplugs = "";
				for (int i = 0; i < setc.plugins.Count; i++)
				{
					tmpplugs = tmpplugs + setc.plugins[i]._path + "?";
				}

				tmpplugs = tmpplugs.TrimEnd( '?' );
				plug.InitializePlugins( tmpplugs );

				for (int i = 0; i < setc.plugins.Count; i++)
				{
					List<ValuesElem> valist = new List<ValuesElem>();
										
					String tmpfieldstr = plug.GetFields( i );
					String[] tmpfields = tmpfieldstr.Split( ';' );

					String tmp1 = "";
					String tmp2 = "";
					String tmpflds = "";

					List<String> vals1 = new List<String>();
					List<String> vals2 = new List<String>();

					//divide into two!

					tmpflds = setc.plugins[i]._fields;

					for (int j = 0; j < tmpfields.Length; j++)
					{
						if (tmpflds.Contains( "*" ) || tmpflds.Contains( ";" + j.ToString() + ";" ))
						{
							tmp1 = plug.GetValueStr( tmpfields[j], firstFile, i );
							tmp1 = tmp1.TrimEnd( Convert.ToChar( (Int32) 0 ) );
							
							vals1.Add(tmp1);							
						}
					}

					for (int j = 0; j < tmpfields.Length; j++)
					{
						if (tmpflds.Contains( "*" ) || tmpflds.Contains( ";" + j.ToString() + ";" ))
						{
							tmp2 = plug.GetValueStr( tmpfields[j], secondFile, i );
							//tmp2 = plug.GetValueStr( plug.plugFieldsList[j]._name, secondFile, setc.plugins[i]._path );
							tmp2 = tmp2.TrimEnd( Convert.ToChar( (Int32) 0 ) );
							
							vals2.Add(tmp2);
						}
					}

					for (int k = 0; k < vals1.Count; k++)
					{
						if (vals1[k].Equals( vals2[k] ))
							{
								if (!setc.showdifferent)
								{
									if (vals1[k].Equals( "" ))
									{
										if (setc.showempty)
										{
											valist.Add( new ValuesElem( tmpfields[k], vals1[k], vals2[k] ) );
										}
									}
									else
									{
										valist.Add( new ValuesElem( tmpfields[k], vals1[k], vals2[k] ) );
									}
								}
							}
							else
							{
								valist.Add( new ValuesElem( tmpfields[k], vals1[k], vals2[k] ) );
							}
					}					

					if (valist.Count > 0)
					{
						SourceGrid.Cells.Views.Cell selView = new SourceGrid.Cells.Views.Cell();
						selView.Font = new Font( this.Font.FontFamily, this.Font.Size, FontStyle.Bold );
						SourceGrid.Cells.Views.Cell selView2 = new SourceGrid.Cells.Views.Cell();
						selView2.ForeColor = Color.Red;

						detailsGrid.Rows.Insert( ind );
						detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.plugins[i]._name );
						detailsGrid[ind, 0].ColumnSpan = 3;
						detailsGrid[ind, 0].View = selView;
						ind++;

						for (int k = 0; k < valist.Count; k++)
						{
							detailsGrid.Rows.Insert( ind );
							detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( valist[k].Field );
							detailsGrid[ind, 1] = new SourceGrid.Cells.Cell( valist[k].Value1 );
							detailsGrid[ind, 2] = new SourceGrid.Cells.Cell( valist[k].Value2 );

							if (!setc.showdifferent && !valist[k].Value1.Equals( valist[k].Value2 ))
							{								
								detailsGrid[ind, 1].View = selView2;
								detailsGrid[ind, 2].View = selView2;								
							}

							ind++;							
						}
					}
					else
					{
						if (setc.showempty)
						{
							SourceGrid.Cells.Views.Cell selView = new SourceGrid.Cells.Views.Cell();
							selView.Font = new Font( this.Font.FontFamily, this.Font.Size, FontStyle.Bold );

							detailsGrid.Rows.Insert( ind );
							detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.plugins[i]._name + setc.GetLangText( "EmptyError" ) );
							detailsGrid[ind, 0].ColumnSpan = 3;
							detailsGrid[ind, 0].View = selView;
						}
					}					
				}

				plug.Free();
			}
			else
			{
				SourceGrid.Cells.Views.Cell selView = new SourceGrid.Cells.Views.Cell();
				selView.Font = new Font( this.Font.FontFamily, this.Font.Size, FontStyle.Bold );

				detailsGrid.Rows.Insert( ind );
				detailsGrid[ind, 0] = new SourceGrid.Cells.Cell( setc.GetLangText( "NoPlugins" ) );
				detailsGrid[ind, 0].ColumnSpan = 3;
				detailsGrid[ind, 0].View = selView;
			}
		}
				
		private void CustomCompareMenu_Click( object sender, EventArgs e )
		{
			ToolStripMenuItem tsmi = (ToolStripMenuItem) sender;
			Comparer cmptool = (Comparer) tsmi.Tag;

			String cmp = cmptool._path;
			String par = cmptool._param;
			par = par.Replace( "%f1", firstFile );
			par = par.Replace( "%f2", secondFile );

			if (File.Exists( cmp ))
			{
				Process proc = new Process();
				proc.StartInfo.FileName = cmp;
				proc.StartInfo.Arguments = par;
				proc.Start();

				if (setc.closecmp)
				{
					this.Close();
				}
			}
		}

		private void grid1_DragEnter( object sender, DragEventArgs e )
		{
			if (e.Data.GetDataPresent( DataFormats.FileDrop, false ) == true)
			{				
				string[] files = (string[]) e.Data.GetData( DataFormats.FileDrop );

				if (GetFilesOnly(files))
				{
					e.Effect = DragDropEffects.All;
				}
				else
				{
					e.Effect = DragDropEffects.None;
				}
			}
		}

		private void grid1_DragDrop( object sender, DragEventArgs e )
		{
			string[] files = (string[]) e.Data.GetData( DataFormats.FileDrop );			

			if (setc.stickmode)
			{				
				detailsGrid.Rows.Clear();
				detailsGrid.Refresh();

				if (files.Length >= 2)
				{
					firstFile = files[0];
					secondFile = files[1];
					this.mode = StartMode.DOUBLE;
					GetDoubleFilesInfo( firstFile, secondFile );
					GenerateCustomCompareMenu();
				}

				if (files.Length == 1)
				{
					switch (mode)
					{
						case StartMode.SINGLE:
							switch (GetDropColumn( e.X, e.Y ))
							{
								case 0:
									secondFile = firstFile;
									firstFile = files[0];
									this.mode = StartMode.DOUBLE;
									GetDoubleFilesInfo( firstFile, secondFile );
									break;
								case 1:
									firstFile = files[0];
									this.mode = StartMode.SINGLE;
									GetSingleFileInfo( firstFile );									
									break;
								case 2:
									secondFile = files[0];
									this.mode = StartMode.DOUBLE;
									GetDoubleFilesInfo( firstFile, secondFile );
									break;
								default:
									break;
							}
							break;
						case StartMode.DOUBLE:
							switch (GetDropColumn( e.X, e.Y ))
							{
								case 1:
									firstFile = files[0];
									this.mode = StartMode.DOUBLE;
									GetDoubleFilesInfo( firstFile, secondFile );
									GenerateCustomCompareMenu();
									break;
								case 2:
									secondFile = files[0];
									this.mode = StartMode.DOUBLE;
									GetDoubleFilesInfo( firstFile, secondFile );
									GenerateCustomCompareMenu();
									break;
								default:
									firstFile = files[0];
									this.mode = StartMode.SINGLE;
									GetSingleFileInfo( firstFile );
									break;
							}
							break;
						default:
							firstFile = files[0];
							this.mode = StartMode.SINGLE;
							GetSingleFileInfo( firstFile );
							break;
					}
				}
			}
			else
			{
				detailsGrid.Rows.Clear();
				detailsGrid.Refresh();

				if (files.Length == 1)
				{
					//detailsTLV.Nodes.Clear();
					firstFile = files[0];
					this.mode = StartMode.SINGLE;
					GetSingleFileInfo( firstFile );
				}

				if (files.Length >= 2)
				{
					firstFile = files[0];
					secondFile = files[1];
					this.mode = StartMode.DOUBLE;
					GetDoubleFilesInfo( firstFile, secondFile );
					GenerateCustomCompareMenu();
				}
			}

			if (setc.remembersize)
			{
				switch (mode)
				{					
					case StartMode.SINGLE:
						detailsGrid.Columns[0].Width = setc.size_label;
						detailsGrid.Columns[1].Width = setc.size_scol;
						break;
					case StartMode.DOUBLE:
						detailsGrid.Columns[0].Width = setc.size_label;
						detailsGrid.Columns[1].Width = setc.size_dcol1;
						detailsGrid.Columns[2].Width = setc.size_dcol2;
						break;				
					default:
						break;
				}
			}

			exportTSDB.Enabled = true;
		}

		public void RefreshForm( object sender )
		{
			Debug.WriteLine( "Start - RefreshForm" );
			setc = new SettingsClass( this.inifile );
			setc.ReadGeneral();
			SetFormLang();
			//CheckCustomInis();
			//FillInisCombo();

			this.TopMost = setc.alwaystop;						

			switch (mode)
			{
				case StartMode.NONE:
					//TODO: Info dialog
					compareTSDB.Enabled = false;
					exportTSDB.Enabled = false;
					stickmodeTSB.Enabled = false;
					stickmodeTSB.CheckState = BooleanToCheckState( setc.stickmode );
					ValidateStickCheck();					
					break;
				case StartMode.SINGLE:
					//detailsTLV.Nodes.Clear();
					GetSingleFileInfo( firstFile );
					compareTSDB.Enabled = false;
					exportTSDB.Enabled = true;
					stickmodeTSB.Enabled = true;
					stickmodeTSB.CheckState = BooleanToCheckState( setc.stickmode );
					ValidateStickCheck();
					break;
				case StartMode.DOUBLE:
					//detailsTLV.Nodes.Clear();
					GetDoubleFilesInfo( firstFile, secondFile );
					GenerateCustomCompareMenu();
					exportTSDB.Enabled = true;
					stickmodeTSB.Enabled = true;
					stickmodeTSB.CheckState = BooleanToCheckState( setc.stickmode );
					ValidateStickCheck();
					break;
				default:
					break;
			}

			Debug.WriteLine( "End - RefreshForm" );
			
		}		

		private void Form2_FormClosing( object sender, FormClosingEventArgs e )
		{
			if (setc.remembersize)
			{
				setc.SetSize( this.Width, this.Height );

				//save label column

				setc.size_label = this.detailsGrid.Columns[0].Width;

				switch (this.mode)
				{
					case StartMode.SINGLE:
						setc.size_scol = this.detailsGrid.Columns[1].Width;
						break;
					case StartMode.DOUBLE:
						setc.size_dcol1 = this.detailsGrid.Columns[1].Width;
						setc.size_dcol2 = this.detailsGrid.Columns[2].Width;
						break;
					default:
						break;
				}

				setc.SetColumns( setc.size_label, setc.size_scol, setc.size_dcol1, setc.size_dcol2 );
			}

			//setc.SetCustomInis();
						
			if (!this._startini.Equals( this.inifile ))
			{
				SettingsClass tmpset = new SettingsClass( this._startini );
				tmpset.ReadGeneral();

				if (tmpset.rememberlastini)
				{
					IniFile tmpini = new IniFile( this._startini );
					SettElem lem = (SettElem) custominisCB.SelectedItem;

					String tmpinif = lem._file;
					tmpinif = tmpinif.Substring( tmpinif.LastIndexOf( '\\' ) + 1 );
					tmpini.WriteValue( "General", "LastIniFile", tmpinif );										
				}				
			}			
		}

		private void General_KeyPress( object sender, KeyPressEventArgs e )
		{			
			if (e.KeyChar == (char) Keys.Escape)
			{
				this.Close();
			}

			if (sender.GetType() == typeof( SourceGrid.Grid ))
			{
				if ((int) e.KeyChar == 3)
				{
					int poz = detailsGrid.Selection.ActivePosition.Row;
										
					if (poz > 0)
					{
						if (this.setc.showthumbs && (poz == 1))
						{
							//TODO: image handling
						}
						else
						{
							String tmp = "";
							tmp = tmp + detailsGrid[poz, 0].Value;

							if (detailsGrid[poz, 0].ColumnSpan <= 1)
							{
								for (int i = 1; i < detailsGrid.ColumnsCount; i++)
								{
									tmp = tmp + "\t" + detailsGrid[poz, i].Value;
								}
							}
							Clipboard.SetText( tmp );
						}						
					}

				}

				if ((int) e.KeyChar == 67)
				{
					int poz = detailsGrid.Selection.ActivePosition.Row;

					if (poz > 0)
					{
						if (this.setc.showthumbs && (poz == 1))
						{
							//TODO: image handling
						}
						else
						{							
							RichTextBox rtb = new RichTextBox();						

							if (detailsGrid[poz, 0].View.Font != null)
							{
								rtb.SelectionFont = new Font( detailsGrid[poz, 0].View.Font, detailsGrid[poz, 0].View.Font.Style );
							}
							else
							{
								rtb.SelectionFont = new Font( detailsGrid.Font, detailsGrid.Font.Style );			
								rtb.SelectionColor = detailsGrid[poz, 0].View.ForeColor;
							}
							rtb.SelectedText += detailsGrid[poz, 0].Value;

							if (detailsGrid[poz, 0].ColumnSpan <= 1)
							{
								for (int i = 1; i < detailsGrid.ColumnsCount; i++)
								{
									if (detailsGrid[poz, i].View.Font != null)
									{
										rtb.SelectionFont = new Font( detailsGrid[poz, i].View.Font, detailsGrid[poz, i].View.Font.Style );									
									}
									else
									{
										rtb.SelectionFont = new Font( detailsGrid.Font, detailsGrid.Font.Style );
										rtb.SelectionColor = detailsGrid[poz, i].View.ForeColor;
									}

									rtb.SelectedText += "\t" + detailsGrid[poz, i].Value;
								}
							}							
							Clipboard.SetText( rtb.Rtf, TextDataFormat.Rtf );
						}
					}
				}
			}		
			
		}
				
		public Boolean CheckStateToBoolean(CheckState val)
		{
			switch (val)
			{
				case CheckState.Checked:
					return true;
					break;				
				default:
					return false;
					break;
			}

			return false;
		}

		public CheckState BooleanToCheckState( Boolean val )
		{
			if (val)
			{
				return CheckState.Checked;
			}

			return CheckState.Unchecked;
		}

		private void ValidateStickCheck()
		{
			switch (stickmodeTSB.CheckState)
			{
				case CheckState.Checked:
					this.stickmodeTSB.Image = global::Comparer.Properties.Resources.ok;
					break;
				default:
					this.stickmodeTSB.Image = global::Comparer.Properties.Resources.cancel;
					break;
			}
		}

		private int GetDropColumn( int x, int y )
		{
			int poz = detailsGrid.Left + this.Left;

			int tmp0 = poz;
			int tmp1 = tmp0 + detailsGrid.Columns[0].Width;
			int tmp2 = tmp1 + detailsGrid.Columns[1].Width;
			int tmp3;			

			if ((x > tmp0) && (x < tmp1))
			{
				return 0;
			}

			if ((x > tmp1) && (x < tmp2))
			{
				return 1;
			}

			if (this.mode == StartMode.DOUBLE)
			{
				tmp3 = tmp2 + detailsGrid.Columns[2].Width;

				if ((x > tmp2) && (x < tmp3))
				{
					return 2;
				}

				if (x > tmp3)
				{
					return 3;
				}
			}
			else
			{
				if (x > tmp2)
				{
					return 2;
				}
			}

			return 0;
		}

		private Boolean GetFilesOnly( string[] files )
		{
			for (int i = 0; i < files.Length; i++)
			{
				if (Directory.Exists(files[i]))
				{
					return false;
				}
			}

			return true;
		}

		private void optionsTSB_Click( object sender, EventArgs e )
		{
			try
			{
				SettingsForm sf = new SettingsForm( this.inifile );
				sf.RefreshForm += new RefreshFormEventHandler( RefreshForm );
				sf.ShowDialog( this );
			}
			catch (Exception exc)
			{
				MessageBox.Show(exc.ToString());
			}
		}

		private void stickmodeTSB_CheckStateChanged( object sender, EventArgs e )
		{
			//MessageBox.Show(stickmodeTSMIToolStripMenuItem.CheckState.ToString());

			switch (stickmodeTSB.CheckState)
			{
				case CheckState.Checked:
					this.stickmodeTSB.Image = global::Comparer.Properties.Resources.ok;
					break;
				default:
					this.stickmodeTSB.Image = global::Comparer.Properties.Resources.cancel;
					break;
			}

			setc.stickmode = CheckStateToBoolean( stickmodeTSB.CheckState );
		}

		private void cSVToolStripMenuItem1_Click( object sender, EventArgs e )
		{
			String tmp = "output";

			switch (mode)
			{				
				case StartMode.SINGLE:
					tmp = NamesForm.CheckVariables( Converters.GetShortFileName(firstFile), setc.outsname );
					break;
				case StartMode.DOUBLE:
					tmp = NamesForm.CheckVariables( Converters.GetShortFileName( firstFile ), Converters.GetShortFileName( secondFile ), setc.outdname );
					break;
				default:
					break;
			}		

			exportSFD.FileName = tmp;
			exportSFD.DefaultExt = "csv";
			exportSFD.Filter =  this.setc.GetLangText("CSVSaveAs") + "|*.csv";
			exportSFD.Title = this.setc.GetLangText( "CSVTitle" );

			try
			{
				if (exportSFD.ShowDialog() == DialogResult.OK)
				{					
					Exporters exp = new Exporters( exportSFD.FileName, detailsGrid, setc );
					exp.ExportCSV();

					if (setc.runassapp)
					{
						Process proc = new Process();
						proc.StartInfo.UseShellExecute = true;
						proc.StartInfo.FileName = exportSFD.FileName;
						proc.Start();
					}

				}
			}
			catch (Exception err)
			{
				//DevAge.Windows.Forms.ErrorDialog.Show( this, err, this.setc.GetLangText( "CSVExportError" ) );
				MessageBox.Show( this.setc.GetLangText( "CSVExportError" ), this.setc.GetLangText( "MsgBoxTitleError" ), MessageBoxButtons.OK, MessageBoxIcon.Error );
			}
		}

		private void hTMLToolStripMenuItem1_Click( object sender, EventArgs e )
		{
			String tmp = "output";

			switch (mode)
			{
				case StartMode.SINGLE:
					tmp = NamesForm.CheckVariables( Converters.GetShortFileName( firstFile ), setc.outsname );
					break;
				case StartMode.DOUBLE:
					tmp = NamesForm.CheckVariables( Converters.GetShortFileName( firstFile ), Converters.GetShortFileName( secondFile ), setc.outdname );
					break;
				default:
					break;
			}

			exportSFD.Title = this.setc.GetLangText( "HTMLTitle" );

			exportSFD.FileName = tmp;
			exportSFD.DefaultExt = "html";
			exportSFD.Filter = this.setc.GetLangText( "HTMLSaveAs" ) + "|*.html";

			try
			{
				if (exportSFD.ShowDialog() == DialogResult.OK)
				{					
					Exporters exp = new Exporters( exportSFD.FileName, detailsGrid, setc );
					exp.ExportHTML();

					if (setc.runassapp)
					{
						Process proc = new Process();
						proc.StartInfo.UseShellExecute = true;
						proc.StartInfo.FileName = exportSFD.FileName;
						proc.Start();
					}
				}

			}
			catch (Exception err)
			{
				//DevAge.Windows.Forms.ErrorDialog.Show( this, err, this.setc.GetLangText( "HTMLExportError" ) );
				MessageBox.Show( this.setc.GetLangText( "HTMLExportError" ), this.setc.GetLangText( "MsgBoxTitleError" ), MessageBoxButtons.OK, MessageBoxIcon.Error );
			}
		}

		private void imageToolStripMenuItem1_Click( object sender, EventArgs e )
		{
			String tmp = "output";

			switch (mode)
			{
				case StartMode.SINGLE:
					tmp = NamesForm.CheckVariables( Converters.GetShortFileName( firstFile ), setc.outsname );
					break;
				case StartMode.DOUBLE:
					tmp = NamesForm.CheckVariables( Converters.GetShortFileName( firstFile ), Converters.GetShortFileName( secondFile ), setc.outdname );
					break;
				default:
					break;
			}

			exportSFD.FileName = tmp;			
			exportSFD.DefaultExt = "bmp";
			exportSFD.Title = this.setc.GetLangText( "ImageTitle" );

			String tmpexp = this.setc.GetLangText( "ImageSaveAsBMP" ) + "|*.bmp|";
			tmpexp = tmpexp + this.setc.GetLangText( "ImageSaveAsJPG" ) + "|*.jpg|";
			tmpexp = tmpexp + this.setc.GetLangText( "ImageSaveAsPNG" ) + "|*.png";
			exportSFD.Filter = tmpexp;
			//this.setc.GetLangText( "ImageExportError" )

			try
			{
				if (exportSFD.ShowDialog() == DialogResult.OK)
				{
					Exporters exp = new Exporters( exportSFD.FileName, detailsGrid, setc );

					switch (this.mode)
					{
						case StartMode.SINGLE:
							exp.ExportImage( exportSFD.FilterIndex, this.firstFile );
							break;
						case StartMode.DOUBLE:
							exp.ExportImage( exportSFD.FilterIndex, this.firstFile, this.secondFile );
							break;						
						default:
							break;
					}

					if (setc.runassapp)
					{						
						Process proc = new Process();
						proc.StartInfo.UseShellExecute = true;
						proc.StartInfo.FileName = exportSFD.FileName;
						proc.Start();
					}
				}
			}
			catch (Exception err)
			{
				//DevAge.Windows.Forms.ErrorDialog.Show( this, err, this.setc.GetLangText( "ImageExportError" ) );
				MessageBox.Show( this.setc.GetLangText( "ImageExportError" ), this.setc.GetLangText( "MsgBoxTitleError" ), MessageBoxButtons.OK, MessageBoxIcon.Error );
			}
		}

		private void CheckWrapping()
		{
			int ind = 1;

			if (setc.showthumbs)
			{
				ind = 2;
			}

			if (setc.wordwrap)
			{
				detailsGrid.AutoStretchRowsToFitHeight = true;

				SourceGrid.Cells.Views.Cell selview = new SourceGrid.Cells.Views.Cell();
				selview.WordWrap = true;

				for (int i = ind; i < detailsGrid.RowsCount; i++)
				{
					for (int j = 0; j < detailsGrid.Columns.Count; j++)
					{
						if (detailsGrid[i, j].View != null)
						{
							detailsGrid[i, j].View.WordWrap = true;
						}
						else
						{
							detailsGrid[i, j].View = selview;
						}

						detailsGrid.Rows.AutoSizeRow( i );
					}
				}
				detailsGrid.AutoStretchRowsToFitHeight = false;
			}
		}

		private void CheckWrapping(int colind)
		{
			int ind = 1;

			if (setc.showthumbs)
			{
				ind = 2;
			}

			if (setc.wordwrap)
			{
				detailsGrid.AutoStretchRowsToFitHeight = true;

				SourceGrid.Cells.Views.Cell selview = new SourceGrid.Cells.Views.Cell();
				selview.WordWrap = true;

				for (int i = ind; i < detailsGrid.RowsCount; i++)
				{					
					if (detailsGrid[i, colind].View != null)
					{
						detailsGrid[i, colind].View.WordWrap = true;
					}
					else
					{
						detailsGrid[i, colind].View = selview;
					}

					detailsGrid.Rows.AutoSizeRow( i );
					
				}

				detailsGrid.AutoStretchRowsToFitHeight = false;
			}
		}

		private void SetFormLang()
		{
			//
			compareTSDB.Text = setc.GetLangText( "TBCompare" );
			compareTSDB.ToolTipText = setc.GetLangText( "TBCompareTip" );
			exportTSDB.Text = setc.GetLangText( "TBExport" );
			exportTSDB.ToolTipText = setc.GetLangText( "TBExportTip" );
			cSVToolStripMenuItem1.Text = setc.GetLangText( "TBExportMenuCSV" );
			hTMLToolStripMenuItem1.Text = setc.GetLangText( "TBExportMenuHTML" );
			imageToolStripMenuItem1.Text = setc.GetLangText( "TBExportMenuImage" );
			stickmodeTSB.Text = setc.GetLangText( "TBStickMode" );
			stickmodeTSB.ToolTipText = setc.GetLangText( "TBStickModeTip" );
			optionsTSB.Text = setc.GetLangText( "TBOptions" );
			optionsTSB.ToolTipText = setc.GetLangText( "TBOptionsTip" );
			configTSL.Text = setc.GetLangText( "TBConfigLabel" );
		}

		private Boolean CheckCustomInis()
		{
			this._custominis = new List<SettElem>();			

			//reading INIs files

			String tmplang = "%APP_PATH%\\configs\\";
			tmplang = setc.AppPathToExe( tmplang );

			this._custominis.Add( new SettElem( this._startini ) );

			if (Directory.Exists( tmplang ))
			{
				//proceed
				DirectoryInfo di = new DirectoryInfo( tmplang );
				FileInfo[] inifiles = di.GetFiles( "*.ini" );

				if (inifiles.Length > 0)
				{
					for (int i = 0; i < inifiles.Length; i++)
					{
						this._custominis.Add( new SettElem( tmplang + inifiles[i].ToString() ) );
					}
					return true;
				}
				else
				{
					return false;
				}
			}

			return false;
		}

		private void FillInisCombo()
		{
			//TODO: fill combo with INIs' names
			custominisCB.Items.Clear();

			foreach (SettElem se in this._custominis)
			{
				if (se._file.Equals( this._startini ))
				{
					se._name = se._name + this.setc.GetLangText("TBConfigCBStart");
					custominisCB.Items.Add( se );
				}
				else
				{
					custominisCB.Items.Add( se );
				}

				if (se._file.Equals( this.setc.inifile.Path ))
				{
					custominisCB.Text = se._name;					
				}
			}
		}

		internal class ValueElem
		{
			private string _field;
			private string _value;

			public ValueElem(String field, String val)
			{
				this._field = field;
				this._value = val;
			}

			public String Field
			{
				get
				{
					return this._field;
				}
				set
				{
					this._field = value;
				}
			}

			public String Value
			{
				get
				{
					return this._value;
				}
				set
				{
					this._value = value;
				}
			}
		}

		internal class ValuesElem
		{
			private string _field;
			private string _value1;
			private string _value2;

			public ValuesElem(String field, String val1, String val2)
			{
				this._field = field;
				this._value1 = val1;
				this._value2 = val2;
			}

			public String Field
			{
				get
				{
					return this._field;
				}
				set
				{
					this._field = value;
				}
			}

			public String Value1
			{
				get
				{
					return this._value1;
				}
				set
				{
					this._value1 = value;
				}
			}

			public String Value2
			{
				get
				{
					return this._value2;
				}
				set
				{
					this._value2 = value;
				}
			}
		}

		private void custominisCB_SelectedIndexChanged( object sender, EventArgs e )
		{
			//
			SettElem lem = (SettElem) custominisCB.SelectedItem;
			this.inifile = lem._file;						

			RefreshForm( this );
		}

		private void removecustominiTSB_Click( object sender, EventArgs e )
		{
			//remove selected
			SettElem lem = (SettElem) custominisCB.SelectedItem;
			String tmpmsg = "";
			String tmptitle = "";

			if (lem == null)
			{
				tmptitle = setc.GetLangText("MsgBoxTitleWarning");
				tmpmsg = setc.GetLangText( "MsgBoxIniNotExist" );

				MessageBox.Show( tmpmsg, tmptitle, MessageBoxButtons.OK, MessageBoxIcon.Warning );
			}
			else
			{
				if (lem._file.Equals( this._startini ))
				{
					tmptitle = setc.GetLangText( "MsgBoxTitleWarning" );
					tmpmsg = setc.GetLangText( "MsgBoxIniCantDelete" );
					MessageBox.Show( tmpmsg, tmptitle, MessageBoxButtons.OK, MessageBoxIcon.Warning );
				}
				else
				{
					//MessageBox.Show( "DELETE");
					File.Delete(lem._file);					
					
					this.inifile = this._startini;
					RefreshForm(this);
					CheckCustomInis();
					FillInisCombo();

					tmptitle = setc.GetLangText( "MsgBoxTitleInfo" );
					tmpmsg = setc.GetLangText( "MsgBoxIniDeleted" );
					tmpmsg = tmpmsg.Replace( "%s", lem._file );
					tmpmsg = tmpmsg.Replace( "\\n", "\r\n" );

					MessageBox.Show( tmpmsg, tmptitle, MessageBoxButtons.OK, MessageBoxIcon.Information );
				}
			}
		}

		private void addcustominiTSB_Click( object sender, EventArgs e )
		{
			//add new ini
			String name = custominisCB.Text;

			String tmpmsg = "";
			String tmptitle = "";
			
			if (IsFileNameCorrect( name ))
			{				
				name = name.Replace( "=", "" );

				String tmpfold = Application.ExecutablePath;
				tmpfold = tmpfold.Remove( inifile.LastIndexOf( "\\" ) + 1 );
				tmpfold = tmpfold + "configs";
				
				if (!Directory.Exists("tmpfold"))
				{
					Directory.CreateDirectory( tmpfold );
				}
								
				IniFile tmpini = new IniFile(GetCorrectFileName(name));
				
				tmpini.WriteValue( "General", "IniName", name );

				tmptitle = setc.GetLangText( "MsgBoxTitleInfo" );
				tmpmsg = setc.GetLangText( "MsgBoxIniCreated" );
				tmpmsg = tmpmsg.Replace( "%s", tmpini.Path );
				tmpmsg = tmpmsg.Replace( "\\n", "\r\n" );

				MessageBox.Show( tmpmsg, tmptitle, MessageBoxButtons.OK, MessageBoxIcon.Information );

				this.inifile=tmpini.Path.ToString();

				RefreshForm( this );
				CheckCustomInis();
				FillInisCombo();
			}
			else
			{
				tmptitle = setc.GetLangText( "MsgBoxTitleWarning" );
				tmpmsg = setc.GetLangText( "MsgBoxIniTrimInfo" );
				tmpmsg = tmpmsg.Replace( "%s", "* . \" / \\ [ ] : ; | = ," );
				tmpmsg = tmpmsg.Replace( "\\n", "\r\n" );
								
				MessageBox.Show( tmpmsg, tmptitle, MessageBoxButtons.OK, MessageBoxIcon.Warning );
			}
		}

		private Boolean IsFileNameCorrect(string tmpname)
		{

			String corfname = GetCorrectFileName( tmpname );
			tmpname = tmpname.Replace( "+", "" ); ;

			if (File.Exists( corfname ))
			{
				return false;
			}
			else
			{
				String temp = "";

				for (int i = 0; i < this.custominisCB.Items.Count; i++)
				{
					SettElem se = (SettElem) this.custominisCB.Items[i];
					temp = se._name;

					if (tmpname.Equals(temp))
					{
						return false;
					}

				}
			}

			return true;
		}		

		private String GetCorrectFileName( string tmpname )
		{
			//replace forbidden or not handled characters
			//space * . " / \ [ ] : ; | = ,
			tmpname = tmpname.Replace( " ", "" );
			tmpname = tmpname.Replace( "*", "" );
			tmpname = tmpname.Replace( "\"", "" );
			tmpname = tmpname.Replace( "/", "" );
			tmpname = tmpname.Replace( "\\", "" );
			tmpname = tmpname.Replace( "[", "" );
			tmpname = tmpname.Replace( "]", "" );
			tmpname = tmpname.Replace( ":", "" );
			tmpname = tmpname.Replace( ";", "" );
			tmpname = tmpname.Replace( "|", "" );
			tmpname = tmpname.Replace( "=", "" );
			tmpname = tmpname.Replace( ",", "" );
			tmpname = tmpname.Replace( "?", "" );

			String tmp = "%APP_PATH%\\configs\\";
			tmp = setc.AppPathToExe( tmp );
			tmp = tmp + tmpname + ".ini";
			
			return tmp;
		}

		private void custominisCB_KeyDown( object sender, KeyEventArgs e )
		{
			if (Control.ModifierKeys == Keys.Alt)
			{
				if (e.KeyCode == Keys.Enter)
				{
					addcustominiTSB_Click( addcustominiTSB, null);
				}

				if (e.KeyCode == Keys.Delete)
				{
					removecustominiTSB_Click( removecustominiTSB, null );
				}
			}
		}		
	}
}
