﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Comparer
{
	public partial class ImagesForm : Form
	{
		private String _firstfile = "d:\\picture.jpg";
		private String _secondfile = "c:\\test\\text.txt";

		private String _imgsname = "";
		private String _imgdname = "";
		private Int32 _imgheight = 100;
		private Int32 _imgwidth = 100;
		private Font _imgfont;
		private Color _imgcolour;
		private SettingsClass _sc;

		public ImagesForm( String imgsname, String imgdname, Int32 imgwidth, Int32 imgheight, Font imgfont, Color imgcolour, SettingsClass sc )
		{
			InitializeComponent();

			this._imgsname = imgsname;
			this._imgdname = imgdname;
			this._imgheight = imgheight;
			this._imgwidth = imgwidth;
			this._imgfont = imgfont;
			this._imgcolour = imgcolour;
			this._sc = sc;
		}

		private void closeBtn_Click( object sender, EventArgs e )
		{
			this.Close();
		}

		private void ImagesForm_Load( object sender, EventArgs e )
		{
			this.singlePB.Width = this._imgwidth;
			this.singlePB.Height = this._imgheight;

			this.singleGB.Width = this.singlePB.Width + 20;
			this.singleGB.Height = this.singlePB.Height + 30;

			this.doublePB.Width = this._imgwidth;
			this.doublePB.Height = this._imgheight;

			this.doubleGB.Top = this.singleGB.Top + this.singleGB.Height + 10;
			this.doubleGB.Width = this.doublePB.Width + 20;
			this.doubleGB.Height = this.doublePB.Height + 30;			

			this.Height = this.doubleGB.Top + this.doubleGB.Height + 30 + this.closeBtn.Height + 20;
			this.Width = this.singleGB.Width + 30;
			singlePB.Image = GetFirstImage();
			doublePB.Image = GetSecondImage();

			//language

			this.Text = this._sc.GetLangText( "ImgShowDlgTitle" );
			singleGB.Text = this._sc.GetLangText( "ImgShowDlgSFile" );
			doubleGB.Text = this._sc.GetLangText( "ImgShowDlgDFile" );
			closeBtn.Text = this._sc.GetLangText( "ImgShowCloseBtn" );
		}

		private Image GetFirstImage()
		{
			Bitmap bmp = new Bitmap( this._imgwidth, this._imgheight );
			Graphics gr = Graphics.FromImage( bmp );
			SolidBrush br = new SolidBrush( this._imgcolour );
			gr.FillRectangle( new SolidBrush( Color.White ), new Rectangle( 0, 0, bmp.Width, bmp.Height ) );

			String tmp = CheckVariables( _firstfile, this._imgsname );

			gr.DrawString( tmp, this._imgfont, br, new Point( 0, 0 ) );
			return bmp;
		}

		private Image GetSecondImage()
		{
			Bitmap bmp = new Bitmap( this._imgwidth, this._imgheight );
			Graphics gr = Graphics.FromImage( bmp );
			SolidBrush br = new SolidBrush( this._imgcolour );
			gr.FillRectangle( new SolidBrush( Color.White ), new Rectangle( 0, 0, bmp.Width, bmp.Height ) );

			String tmp = CheckVariables( _firstfile, _secondfile, this._imgdname );
						
			gr.DrawString( tmp, this._imgfont, br, new Point( 0, 0 ) );
			return bmp;
		}

		public static String CheckVariables( String filename, String vars )
		{
			String tmp = vars;

			if (vars.Contains( "%fs" ))
			{
				tmp = tmp.Replace( "%fs", ShortFile( filename ) );
			}

			if (vars.Contains( "%ff" ))
			{
				tmp = tmp.Replace( "%ff", FullFile( filename ) );
			}

			if (vars.Contains( "%df" ))
			{
				tmp = FormattedDate( tmp );
			}

			if (vars.Contains( "\\n" ))
			{
				tmp = tmp.Replace( "\\n", Environment.NewLine );
			}

			if (vars.Contains( "\\t" ))
			{
				tmp = tmp.Replace( "\\t", "\t" );
			}

			if (vars.Contains( "%fp" ))
			{
				tmp = tmp.Replace( "%fp", FullPath(filename) );
			}

			return tmp;
		}

		public static String CheckVariables( String firstfile, String secondfile, String vars )
		{
			String tmp = vars;

			if (vars.Contains( "%f1s" ))
			{
				tmp = tmp.Replace( "%f1s", ShortFile( firstfile ) );
			}

			if (vars.Contains( "%f2s" ))
			{
				tmp = tmp.Replace( "%f2s", ShortFile( secondfile ) );
			}

			if (vars.Contains( "%f1f" ))
			{
				tmp = tmp.Replace( "%f1f", FullFile( firstfile ) );
			}

			if (vars.Contains( "%f2f" ))
			{
				tmp = tmp.Replace( "%f2f", FullFile( secondfile ) );
			}

			if (vars.Contains( "%df" ))
			{
				tmp = FormattedDate( tmp );
			}

			if (vars.Contains( "\\n" ))
			{
				tmp = tmp.Replace( "\\n", Environment.NewLine );
			}

			if (vars.Contains( "\\t" ))
			{
				tmp = tmp.Replace( "\\t", "\t" );
			}
			if (vars.Contains( "%f1p" ))
			{
				tmp = tmp.Replace( "%f1p", FullPath( firstfile ) );
			}

			if (vars.Contains( "%f2p" ))
			{
				tmp = tmp.Replace( "%f2p", FullPath( secondfile ) );
			}

			return tmp;
		}

		public static String ShortFile( String filename )
		{
			String tmp = filename;
			tmp = tmp.Substring( tmp.LastIndexOf( '\\' ) + 1 );
			tmp = tmp.Substring( 0, tmp.LastIndexOf( '.' ) );

			return tmp;
		}

		public static String FullFile( String filename )
		{
			String tmp = filename;
			tmp = tmp.Substring( tmp.LastIndexOf( '\\' ) + 1 );			

			return tmp;
		}

		public static String FullPath( String filename )
		{
			String tmp = filename;
			tmp = tmp.Substring( 0, tmp.LastIndexOf( '\\' ) + 1);

			return tmp;
		}

		public static String FormattedDate( String vars )
		{
			String tmp = vars.Substring( vars.IndexOf( "%df" ) + 3, vars.Length - vars.IndexOf( "%df" ) - 3 );
			String tmpfor = "";
			String tmpdat = "";

			if (!tmp.Equals( "" ) && tmp[0].Equals( '(' ))
			{
				if (tmp.Contains( ")" ))
				{
					tmpfor = tmp.Substring( 1, tmp.IndexOf( ')' ) - 1 );

					try
					{
						tmpdat = DateTime.Now.Date.ToString( tmpfor );
					}
					catch (Exception exc)
					{
						tmpdat = "<ERRDATE>";
					}

					tmp = vars.Replace( "%df(" + tmpfor + ")", tmpdat );
				}
			}
			else
			{
				tmp = vars;
			}

			return tmp;
		}
	}
}
