﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
//using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO;
using System.Text.RegularExpressions;

namespace Comparer
{
	public delegate void MyFieldsEventHandler( object sender, String fieldstr, int indeks, Size sajz );
		

	public partial class SettingsForm : Form
	{
		private SettingsClass sc;
		private String _inifile;
		private event RefreshFormEventHandler refreshForm;
		private int wdxind;
		private int wlxind;
		private int compind;
		private Boolean edmode;//help for removing lists items
		private Color _imgcolour;
		private List<LangElem> _langs;

		public SettingsForm(string inifile)
		{
			InitializeComponent();
			this._inifile = inifile;
			wdxind = -1;
			wlxind = -1;
			compind = -1;
			edmode = false;
		}

		private void SettingsForm_Resize( object sender, EventArgs e )
		{
			panel1.Height = this.Height - panel2.Height - 20;
		}

		private void SettingsForm_Load( object sender, EventArgs e )
		{			
			if (File.Exists( this._inifile ))
			{
				if (File.Exists( this._inifile + ".bak" ))
				{
					File.Delete( this._inifile + ".bak" );
				}

				File.Copy( this._inifile, this._inifile + ".bak" );
			}

			sc = new SettingsClass( this._inifile );
			sc.plugins = new List<Plug>();
			sc.wlxplugins = new List<WLXPlug>();

			sc.ReadGeneral();			
			
			sizeCB.Checked = sc.remembersize;			
			closecmpCB.Checked = sc.closecmp;
			ontopCB.Checked = sc.alwaystop;
			
			showemptyCB.Checked = sc.showempty;
			showdifferentCB.Checked = sc.showdifferent;
			showthumbsCB.Checked = sc.showthumbs;
			
			heightLabel.Enabled = showthumbsCB.Checked;
			showiconsCB.Enabled = showthumbsCB.Checked;
			showiconsCB.Checked = sc.showicons;
			heightNUD.Enabled = showthumbsCB.Checked;
			heightNUD.Value = sc.thumbsheight;

			stickmodeCB.Checked = sc.stickmode;

			wordwrapCB.Checked = sc.wordwrap;

			String tmp = sc.plugsdir;
			tmp = sc.ExePathToAppPath( tmp );

			defplugpathTB.Text = tmp;

			singlenameTB.Text = sc.outsname;
			doublenameTB.Text = sc.outdname;

			quotesCB.Checked = sc.surquotes;
			separatorTB.Text = sc.sepchar;

			//image header
			showimgheadCB.Checked = sc.imgshowhead;
			singleimageLabel.Enabled = showimgheadCB.Checked;
			doubleimageLabel.Enabled = showimgheadCB.Checked;
			singleimageTB.Enabled = showimgheadCB.Checked;
			doubleimageTB.Enabled = showimgheadCB.Checked;
			singleimageBtn.Enabled = showimgheadCB.Checked;
			doubleimageBtn.Enabled = showimgheadCB.Checked;
			showimageBtn.Enabled = showimgheadCB.Checked;
			heightimageLabel.Enabled = showimgheadCB.Checked;
			imgheightNUD.Enabled = showimgheadCB.Checked;
			widthimageLabel.Enabled = showimgheadCB.Checked;
			imgwidthNUD.Enabled = showimgheadCB.Checked;
			imgfontLabel.Enabled = showimgheadCB.Checked;
			imagefontLabel.Enabled = showimgheadCB.Checked;
			imagefontBtn.Enabled = showimgheadCB.Checked;
			imgcolourLabel.Enabled = showimgheadCB.Checked;
			imagecolourLabel.Enabled = showimgheadCB.Checked;
			imagecolourBtn.Enabled = showimgheadCB.Checked;

			singleimageTB.Text = sc.imgshead;
			doubleimageTB.Text = sc.imgdhead;
			imgheightNUD.Value = sc.imgheadheight;
			imgwidthNUD.Value = sc.imgheadwidth;

			imagefontLabel.Text = sc.imgfontname.ToString() + ", " + sc.imgfontstyle.ToString() + ", " + sc.imgfontsize.ToString(); ;
					
	
			imgcolourCD.Color = sc.imgcolour;
			imagecolourLabel.Text = "RGB ( " + imgcolourCD.Color.R.ToString() + ", " + imgcolourCD.Color.G.ToString() + ", " + imgcolourCD.Color.B.ToString() + " )";
			imagecolourLabel.ForeColor = imgcolourCD.Color;

			runappCB.Checked = sc.runassapp;
			
			sc.ReadAllPlugins();
			sc.ReadAllWLXPlugins();
			sc.ReadAllComparers();
			RefreshList(0);
			RefreshWLXList( 0 );
			RefreshCompList( 0 );
			RefreshWDXTab();
			RefreshWLXTab();
			RefreshCompTab();

			// set fontFialog
			TypeConverter converter = TypeDescriptor.GetConverter( typeof( FontStyle ) );
			FontStyle fs = (FontStyle) converter.ConvertFromString( sc.imgfontstyle );

			imagefontFontD.Font = new Font( new FontFamily( sc.imgfontname ), sc.imgfontsize, fs );

			//language part
			SetFormLang();
			CheckLangs();
			FillLangsCombo(sc.langfile);
			
			//TODO: set langs combobox
		}		

		private void cancelBtn_Click( object sender, EventArgs e )
		{
			if (File.Exists( this._inifile + ".bak" ))
			{
				File.Delete( this._inifile );
				File.Move( this._inifile + ".bak", this._inifile );
			}
			this.Close();
		}

		private void defaultBtn_Click( object sender, EventArgs e )
		{
			sc.resetsize = true;			
		}		

		private void pluginsLB_SelectedIndexChanged( object sender, EventArgs e )
		{
			if (!edmode)
			{
				if ((wdxind >= 0) && (wdxind < pluginsLB.Items.Count))
				{
					((Plug) pluginsLB.Items[wdxind])._path = pluginTB.Text;
					((Plug) pluginsLB.Items[wdxind])._exts = extensionsTB.Text;
				}				
			}
			else
			{
				edmode = false;			
			}

			if (pluginsLB.SelectedIndex >= 0)
			{
				if (!((Plug) pluginsLB.Items[pluginsLB.SelectedIndex])._path.Equals( "\\<TEMPORARY>" ))
				{
					String tmp = ((Plug) pluginsLB.Items[pluginsLB.SelectedIndex])._path;
					tmp = sc.PluginsToPlugPath( tmp );
					tmp = sc.ExePathToAppPath( tmp );

					pluginTB.Text = tmp;

					extensionsTB.Text = ((Plug) pluginsLB.Items[pluginsLB.SelectedIndex])._exts;
				}
				else
				{
					pluginTB.Text = "";
					extensionsTB.Text = "";
				}
			}

			wdxind = pluginsLB.SelectedIndex;
		}		

		private void pluginBtn_Click( object sender, EventArgs e )
		{
			openFD.Title = sc.GetLangText( "WDXPluginDlgTitle" );
			openFD.Filter = sc.GetLangText( "WDXPluginDlgType" ) + "|*.wdx";
			openFD.FileName = "*.wdx";
			openFD.InitialDirectory = "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}";

			if (!pluginTB.Text.Equals( "" ) && (pluginsLB.SelectedIndex >= 0))
			{
				String tmp = ((Plug) pluginsLB.Items[pluginsLB.SelectedIndex])._path;
				tmp = sc.PlugPathToPlugins( tmp );
				tmp = sc.AppPathToExe( tmp );

				openFD.InitialDirectory = tmp;				
			}			

			if (openFD.ShowDialog() == DialogResult.OK)
			{
				try
				{
					FileInfo tmpfile = new FileInfo( openFD.FileName.ToString() );

					if (tmpfile.Extension.ToUpper().Equals( ".WDX" ))
					{						
						String tmpext = ((Plug) pluginsLB.Items[pluginsLB.SelectedIndex])._exts;
						String tmp2 = tmpfile.FullName;
						tmp2 = sc.PluginsToPlugPath( tmp2 );
						tmp2 = sc.ExePathToAppPath( tmp2 );
						String tmpfld = ((Plug) pluginsLB.Items[pluginsLB.SelectedIndex])._fields;
						((Plug) pluginsLB.Items[pluginsLB.SelectedIndex])._path = tmp2;
						((Plug) pluginsLB.Items[pluginsLB.SelectedIndex]).ValName();						
						
						pluginTB.Text = tmp2;
						extensionsTB.Text = tmpext;
						ReloadWDXList();
					}
				}
				catch (Exception ex)
				{
					//
				}
			}
		}

		private void ReloadWDXList()
		{
			List<Plug> lista = new List<Plug>();
			int sel = pluginsLB.SelectedIndex;

			for (int i = 0; i < pluginsLB.Items.Count; i++)
			{
				lista.Add( (Plug) pluginsLB.Items[i] );
			}

			pluginsLB.Items.Clear();

			for (int j = 0; j < lista.Count; j++)
			{
				pluginsLB.Items.Add( new Plug(lista[j]._path, lista[j]._exts, lista[j]._fields) ); ;
			}
			pluginsLB.SelectedIndex = sel;
		}
				
		private void executableBtn_Click( object sender, EventArgs e )
		{
			openFD.Title = this.sc.GetLangText( "ExecutableDlgTitle" );// "Choose compare tool EXE file";
			openFD.Filter = this.sc.GetLangText( "ExecutableDlgType" ) + "|*.exe";
			openFD.FileName = "*.exe";
			openFD.InitialDirectory = "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}";

			if (!executableTB.Text.Equals( "" ) && (comparersLB.SelectedIndex >= 0))
			{
				String tmp = ((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._path;
				tmp = sc.PlugPathToPlugins( tmp );
				tmp = sc.AppPathToExe( tmp );

				openFD.InitialDirectory = tmp;
			}

			if (openFD.ShowDialog() == DialogResult.OK)
			{
				/*
				String tmp2 = openFD.FileName.ToString();
				tmp2 = sc.PluginsToPlugPath( tmp2 );
				tmp2 = sc.ExePathToAppPath( tmp2 );

				executableTB.Text = tmp2;
				*/

				FileInfo tmpfile = new FileInfo( openFD.FileName.ToString() );

				try
				{
					if (tmpfile.Extension.ToUpper().Equals( ".EXE" ))
					{
						String tmpext = ((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._exts;
						String tmppar = ((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._param;
						String tmp2 = tmpfile.FullName;
						tmp2 = sc.PluginsToPlugPath( tmp2 );
						tmp2 = sc.ExePathToAppPath( tmp2 );

						((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._path = tmp2;
						((Comparer) comparersLB.Items[comparersLB.SelectedIndex]).ValName();

						executableTB.Text = tmp2;
						compextensionsTB.Text = tmpext;
						paramTB.Text = tmppar;
						ReloadCompList();
					}
				}
				catch (Exception ex)
				{
					//
				}
			}			
		}

		private void ReloadCompList()
		{
			List<Comparer> lista = new List<Comparer>();
			int sel = comparersLB.SelectedIndex;

			for (int i = 0; i < comparersLB.Items.Count; i++)
			{
				lista.Add( (Comparer) comparersLB.Items[i] );
			}

			comparersLB.Items.Clear();

			for (int j = 0; j < lista.Count; j++)
			{
				comparersLB.Items.Add( new Comparer(lista[j]._path, lista[j]._exts, lista[j]._param ) ); ;
			}

			comparersLB.SelectedIndex = sel;
		}

		private void addBtn_Click( object sender, EventArgs e )
		{
			openFD.Title = sc.GetLangText( "WDXPluginDlgTitle" );
			openFD.Filter = sc.GetLangText( "WDXPluginDlgType" ) + "|*.wdx";
			openFD.FileName = "*.wdx";			
			openFD.InitialDirectory = "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}";
			
			if (openFD.ShowDialog() == DialogResult.OK)
			{
				try
				{
					FileInfo tmpfile = new FileInfo( openFD.FileName.ToString() );

					if (tmpfile.Extension.ToUpper().Equals( ".WDX" ))
					{
						String tmpext = GetWDXExtensions( openFD.FileName.ToString() );
						String tmp2 = tmpfile.FullName;
						
						if (tmpext.Equals( "" ) || tmpext.Length == 0 )
						{
							tmpext = "*.*";
						}
						
						pluginsLB.Items.Add( new Plug( tmpfile.FullName, tmpext, "*" ) );
						pluginsLB.SelectedIndex = pluginsLB.Items.Count - 1;
					}
				}
				catch (Exception ex)
				{
					//
				}
				RefreshWDXTab();
			}
		}

		private void removeBtn_Click( object sender, EventArgs e )
		{
			int sel = pluginsLB.SelectedIndex;
			edmode = true;

			if (pluginsLB.SelectedIndex >= 0)
			{
				pluginsLB.Items.RemoveAt( pluginsLB.SelectedIndex );
			}

			if (pluginsLB.Items.Count > 0)
			{
				if (sel == 0)
				{
					pluginsLB.SelectedIndex = 0;
				}
				else
				{
					if (sel >= pluginsLB.Items.Count)
					{
						pluginsLB.SelectedIndex = pluginsLB.Items.Count - 1;
					}
					else
					{
						pluginsLB.SelectedIndex = sel;
					}
				}
			}
			else
			{
				pluginTB.Text = "";
				extensionsTB.Text = "";
			}

			RefreshWDXTab();
		}

		private void upBtn_Click( object sender, EventArgs e )
		{
			edmode = true;
			int sel = pluginsLB.SelectedIndex;

			if (sel > 0)
			{
				Plug tmp = (Plug) pluginsLB.Items[pluginsLB.SelectedIndex - 1];
				pluginsLB.Items[pluginsLB.SelectedIndex - 1] = (Plug) pluginsLB.Items[pluginsLB.SelectedIndex];
				pluginsLB.Items[pluginsLB.SelectedIndex] = tmp;
				pluginsLB.SelectedIndex = sel - 1;
			}
		}

		private void downBtn_Click( object sender, EventArgs e )
		{
			edmode = true;
			int sel = pluginsLB.SelectedIndex;

			if (sel < pluginsLB.Items.Count - 1)
			{
				Plug tmp = (Plug) pluginsLB.Items[pluginsLB.SelectedIndex+1];
				pluginsLB.Items[pluginsLB.SelectedIndex + 1] = (Plug) pluginsLB.Items[pluginsLB.SelectedIndex];
				pluginsLB.Items[pluginsLB.SelectedIndex] = tmp;
				pluginsLB.SelectedIndex = sel + 1;
			}
		}

		private void RefreshList(Int32 sel)
		{
			pluginsLB.Items.Clear();

			if (sc.plugins.Count > 0)
			{
				for (int i = 0; i < sc.plugins.Count; i++)
				{
					pluginsLB.Items.Add( sc.plugins[i] );
				}

				pluginsLB.SelectedIndex = sel;

				pluginBtn.Enabled = true;
				pluginTB.Enabled = true;
				extensionsTB.Enabled = true;
			}
		}

		private void RefreshWLXList( Int32 sel )
		{
			wlxpluginsLB.Items.Clear();

			if (sc.wlxplugins.Count > 0)
			{
				for (int i = 0; i < sc.wlxplugins.Count; i++)
				{
					wlxpluginsLB.Items.Add( sc.wlxplugins[i] );
				}

				wlxpluginsLB.SelectedIndex = sel;

				wlxpluginBtn.Enabled = true;
				wlxpluginTB.Enabled = true;
				wlxextensionsTB.Enabled = true;
			}
		}

		private void RefreshCompList( Int32 sel )
		{
			comparersLB.Items.Clear();

			if (sc.comparers.Count > 0)
			{
				for (int i = 0; i < sc.comparers.Count; i++)
				{
					comparersLB.Items.Add( sc.comparers[i] );
				}

				comparersLB.SelectedIndex = sel;
			}
		}		
			
		private void Comparer_DragEnter( object sender, DragEventArgs e )
		{
			if (e.Data.GetDataPresent( DataFormats.FileDrop, false ) == true)
			{				
				e.Effect = DragDropEffects.All;
			}
		}

		private void Comparer_DragDrop( object sender, DragEventArgs e )
		{			
			string[] files = (string[]) e.Data.GetData( DataFormats.FileDrop );
			FileInfo tmpfile = new FileInfo( files[0] );
			
			try
			{
				if (tmpfile.Extension.ToUpper().Equals( ".EXE" ))
				{
					String tmpext = ((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._exts;
					String tmppar = ((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._param;
					String tmp2 = tmpfile.FullName;
					tmp2 = sc.PluginsToPlugPath( tmp2 );
					tmp2 = sc.ExePathToAppPath( tmp2 );

					((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._path = tmp2;
					((Comparer) comparersLB.Items[comparersLB.SelectedIndex]).ValName();

					executableTB.Text = tmp2;
					compextensionsTB.Text = tmpext;
					paramTB.Text = tmppar;
					ReloadCompList();
				}
			}
			catch (Exception ex)
			{
				//
			}
		}

		private void Plugin_DragDrop( object sender, DragEventArgs e )
		{
			string[] files = (string[]) e.Data.GetData( DataFormats.FileDrop );

			try
			{
				FileInfo tmpfile = new FileInfo( files[0] );

				if (tmpfile.Extension.ToUpper().Equals( ".WDX" ))
				{					
					String tmpext = ((Plug) pluginsLB.Items[pluginsLB.SelectedIndex])._exts;
					String tmp2 = tmpfile.FullName;
					tmp2 = sc.PluginsToPlugPath( tmp2 );
					tmp2 = sc.ExePathToAppPath( tmp2 );
					String tmpfld = ((Plug) pluginsLB.Items[pluginsLB.SelectedIndex])._fields;
					((Plug) pluginsLB.Items[pluginsLB.SelectedIndex])._path = tmp2;
					((Plug) pluginsLB.Items[pluginsLB.SelectedIndex]).ValName();

					pluginTB.Text = tmp2;
					extensionsTB.Text = tmpext;
					ReloadWDXList();
				}				
			}
			catch (Exception ex)
			{
				//
			}			
		}

		private void okBtn_Click( object sender, EventArgs e )
		{
			Debug.WriteLine( "Settings - OK click" );
			Debug.WriteLine( "Assigning currently selected WDX" );
			//WDX Apply button part
			if (pluginsLB.SelectedIndex >= 0)
			{
				((Plug) pluginsLB.Items[pluginsLB.SelectedIndex])._path = pluginTB.Text;
				((Plug) pluginsLB.Items[pluginsLB.SelectedIndex]).ValName();
				((Plug) pluginsLB.Items[pluginsLB.SelectedIndex])._exts = extensionsTB.Text;
			}


			sc.plugins = new List<Plug>();
			Debug.WriteLine( "Start iterating list control - WDX" );
			for (int i = 0; i < pluginsLB.Items.Count; i++)
			{
				String tmpplug = ((Plug) pluginsLB.Items[i])._path;

				String tmpexts = ((Plug) pluginsLB.Items[i])._exts;
				String tmpfields = ((Plug) pluginsLB.Items[i])._fields;

				sc.plugins.Add( new Plug( tmpplug, tmpexts, tmpfields ) );
			}

			Debug.WriteLine( "End iterating list control - WDX" );
			
			//WLX Apply button part
			Debug.WriteLine( "Assigning currently selected WLX" );
			if (wlxpluginsLB.SelectedIndex >= 0)
			{
				((WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex])._path = wlxpluginTB.Text;
				((WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex]).ValName();
				((WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex])._exts = wlxextensionsTB.Text;
			}

			sc.wlxplugins = new List<WLXPlug>();
			Debug.WriteLine( "Start iterating list control - WLX" );
			
			for (int j = 0; j < wlxpluginsLB.Items.Count; j++)
			{
				String tmpplug = ((WLXPlug) wlxpluginsLB.Items[j])._path;
				String tmpexts = ((WLXPlug) wlxpluginsLB.Items[j])._exts;

				sc.wlxplugins.Add( new WLXPlug( tmpplug, tmpexts ) );
			}
			Debug.WriteLine( "End iterating list control - WLX" );
			
			//Comparers Apply btn
			Debug.WriteLine( "Assigning currently selected - Comparers" );
			if (comparersLB.SelectedIndex >= 0)
			{
				((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._path = executableTB.Text;
				((Comparer) comparersLB.Items[comparersLB.SelectedIndex]).ValName();
				((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._param = paramTB.Text;
				((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._exts = compextensionsTB.Text;
			}
			
			sc.comparers = new List<Comparer>();
			Debug.WriteLine( "Start iterating list control - Comparers" );
			for (int i = 0; i < comparersLB.Items.Count; i++)
			{
				String tmpplug = ((Comparer) comparersLB.Items[i])._path;
				String tmpexts = ((Comparer) comparersLB.Items[i])._exts;
				String tmppar = ((Comparer) comparersLB.Items[i])._param;

				sc.comparers.Add( new Comparer( tmpplug, tmpexts, tmppar ) );
			}
			Debug.WriteLine( "End iterating list control - Comparers" );

			//save general		
			Debug.WriteLine( "Start saving general options" );
			sc.showempty = showemptyCB.Checked;
			sc.showdifferent = showdifferentCB.Checked;
			sc.remembersize = sizeCB.Checked;
			sc.showthumbs = showthumbsCB.Checked;
			sc.thumbsheight = (Int32) heightNUD.Value;
			sc.showicons = showiconsCB.Checked;
			String tmp = defplugpathTB.Text;
			tmp = sc.AppPathToExe( tmp );
			sc.plugsdir = tmp;
			sc.stickmode = stickmodeCB.Checked;
			sc.wordwrap = wordwrapCB.Checked;
			sc.outsname = NamesForm.ReplaceInvalidChars(singlenameTB.Text);
			sc.outdname = NamesForm.ReplaceInvalidChars(doublenameTB.Text);
			sc.surquotes = quotesCB.Checked;

			if (separatorTB.Text.Equals( "" ))
			{
				sc.sepchar = ";";
			}
			else
			{
				sc.sepchar = separatorTB.Text;
			}
			Debug.WriteLine( "End saving general options" );
			Debug.WriteLine( "Start saving image options" );
			//image header
			sc.imgshowhead = showimgheadCB.Checked;
			if (showimgheadCB.Checked)
			{				
				sc.imgheadheight = (Int32) imgheightNUD.Value;
				sc.imgheadwidth = (Int32) imgwidthNUD.Value;
				sc.imgfontname = imagefontFontD.Font.Name.ToString();
				sc.imgfontsize = (Int32) imagefontFontD.Font.Size;
				sc.imgfontstyle = imagefontFontD.Font.Style.ToString();
				sc.imgshead = singleimageTB.Text;
				sc.imgdhead = doubleimageTB.Text;
				sc.imgcolour = imgcolourCD.Color;
			}

			sc.runassapp = runappCB.Checked;
			Debug.WriteLine( "End saving general options" );
			Debug.WriteLine( "Start saving language options" );
			//language
			sc.langfile = ((LangElem) langCB.SelectedItem)._file;

			sc.SaveGeneral();
			Debug.WriteLine( "All general options - SAVED to ini" );
			
			Debug.WriteLine( "Start saving plugins" );
			//save plugs
			sc.SavePlugins();
			sc.SaveWLXPlugins();
			Debug.WriteLine( "End saving plugins" );
			Debug.WriteLine( "Start saving comparers" );
			sc.SaveComparers();
			Debug.WriteLine( "End saving plugins" );
			
			if (File.Exists( this._inifile + ".bak" ))
			{
				File.Delete( this._inifile + ".bak" );
			}

			Debug.WriteLine( "Start refreshing Form2" );
			OnRefreshForm();
			Debug.WriteLine( "End refreshing Form2" );

			Debug.WriteLine( "End of OK click - closing now" );
			
			this.Close();
		}

		private void closecmpCB_CheckStateChanged( object sender, EventArgs e )
		{
			if (closecmpCB.Checked)
			{
				sc.closecmp = true;
			}
			else
			{
				sc.closecmp = false;
			}
		}

		private void ontopCB_CheckStateChanged( object sender, EventArgs e )
		{
			if (ontopCB.Checked)
			{
				sc.alwaystop = true;
			}
			else
			{
				sc.alwaystop = false;
			}
		}

		private void pluginsLB_DragDrop( object sender, DragEventArgs e )
		{
			string[] files = (string[]) e.Data.GetData( DataFormats.FileDrop );

			foreach (string plugdrop in files)
			{
				FileInfo tmpfile = new FileInfo( plugdrop );

				if (tmpfile.Extension.ToUpper().Equals( ".WDX" ))
				{
					String tmpext = GetWDXExtensions( plugdrop );
															
					pluginsLB.Items.Add( new Plug( plugdrop, tmpext, "*" ) );
					pluginsLB.SelectedIndex = pluginsLB.Items.Count - 1;
					extensionsTB.Text = tmpext;
				}
			}
			RefreshWDXTab();
		}

		public String GetWDXExtensions(String filename)
		{
			//
			String tmpstr = "*";

			try
			{				
				//testing detect string

				PluginProxy pl = new PluginProxy();//changed for wdx_proxy!
				pl.Load();//changed for wdx_proxy!				
				pl.InitializePlugins( filename );
				pl.GetFields( 0 );//changed for wdx_proxy!
				//pl.Load( tmpfile.FullName );

				tmpstr = pl.GetDetect(0);//changed for wdx_proxy!
				tmpstr = Regex.Replace( tmpstr, "EXT=\"(.{2,6})\"", "*.$1", RegexOptions.IgnoreCase );
				tmpstr = Regex.Replace( tmpstr, "[(](.{4,8})[)]", "$1", RegexOptions.IgnoreCase );
				tmpstr = tmpstr.Replace( "|", " " );
				tmpstr = Regex.Replace( tmpstr, "([ ]{2,})", " ", RegexOptions.IgnoreCase );					
				pl.Free();

				if ( tmpstr.Contains( ((char) 0).ToString()) )
				{
					if (tmpstr[0] == (char) 0)
					{
						tmpstr = " ";
					}
				}
			}
			catch (Exception ex)
			{				
				tmpstr = "???";//error identyficator!
			}

			if (tmpstr.Length == 0)
			{
				tmpstr = "*";
			}

			return tmpstr;
		}

		private void wlxpluginsLB_SelectedIndexChanged( object sender, EventArgs e )
		{
			if (!edmode)
			{
				if ((wlxind >= 0) && (wlxind < wlxpluginsLB.Items.Count))
				{
					((WLXPlug) wlxpluginsLB.Items[wlxind])._path = wlxpluginTB.Text;
					((WLXPlug) wlxpluginsLB.Items[wlxind])._exts = wlxextensionsTB.Text;
				}				
			}
			else
			{
				edmode = false;
			}

			if (wlxpluginsLB.SelectedIndex >= 0)
			{
				if (!((WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex])._path.Equals( "\\<TEMPORARY>" ))
				{
					String tmp = ((WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex])._path;
					tmp = sc.PluginsToPlugPath( tmp );
					tmp = sc.ExePathToAppPath( tmp );

					wlxpluginTB.Text = tmp;
					wlxextensionsTB.Text = ((WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex])._exts;
				}
				else
				{
					wlxpluginTB.Text = "";
					wlxextensionsTB.Text = "";
				}
			}

			wlxind = wlxpluginsLB.SelectedIndex;			
		}

		private void wlxpluginsLB_DragDrop( object sender, DragEventArgs e )
		{
			string[] files = (string[]) e.Data.GetData( DataFormats.FileDrop );
			
			foreach (string plugdrop in files)
			{
				FileInfo tmpfile = new FileInfo( plugdrop );

				if (tmpfile.Extension.ToUpper().Equals( ".WLX" ))
				{
					String tmpext = GetWLXExtensions( plugdrop );					

					wlxpluginsLB.Items.Add( new WLXPlug( plugdrop, tmpext ) );
					wlxpluginsLB.SelectedIndex = wlxpluginsLB.Items.Count - 1;
					wlxextensionsTB.Text = tmpext;
				}
				
			}
			
			RefreshWLXTab();
		}

		public String GetWLXExtensions( String filename )
		{
			//
			String tmpstr = "";

			try
			{
				//FileInfo tmpfile = new FileInfo( filename );

				WLXPlugin pl = new WLXPlugin();
				pl.Load( filename );//tmpfile.FullName );

				tmpstr = pl.GetDetect();
				tmpstr = Regex.Replace( tmpstr, "EXT=\"(.{2,6})\"", "*.$1", RegexOptions.IgnoreCase );
				tmpstr = Regex.Replace( tmpstr, "[(](.{4,8})[)]", "$1", RegexOptions.IgnoreCase );
				tmpstr = tmpstr.Replace( "|", " " );
				tmpstr = Regex.Replace( tmpstr, "([ ]{2,})", " ", RegexOptions.IgnoreCase );
				pl.Free();

				if (tmpstr.Contains( ((char) 0).ToString() ))
				{
					if (tmpstr[0] == (char) 0)
					{
						return "";
					}
				}

				return tmpstr;				
			}
			catch (Exception ex)
			{
				tmpstr = "???";
			}
			return tmpstr;
		}

		private void wlxpluginTB_DragDrop( object sender, DragEventArgs e )
		{
			string[] files = (string[]) e.Data.GetData( DataFormats.FileDrop );
			try
			{
				FileInfo tmpfile = new FileInfo( files[0] );

				if (tmpfile.Extension.ToUpper().Equals( ".WLX" ))
				{
					String tmpext = ((WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex])._exts;
					String tmp2 = tmpfile.FullName;
					tmp2 = sc.PluginsToPlugPath( tmp2 );
					tmp2 = sc.ExePathToAppPath( tmp2 );					
					((WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex])._path = tmp2;
					((WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex]).ValName();

					wlxpluginTB.Text = tmp2;
					wlxextensionsTB.Text = tmpext;
					ReloadWLXList();
				}
			}
			catch (Exception ex)
			{
				//
			}		
		}

		private void wlxpluginBtn_Click( object sender, EventArgs e )
		{
			openFD.Title = this.sc.GetLangText( "WLXPluginDlgTitle" );//"Choose WLX lister plugin";
			openFD.Filter = this.sc.GetLangText( "WLXPluginDlgType" ) + "|*.wlx";
			openFD.FileName = "*.wlx";
			openFD.InitialDirectory = "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}";

			if (!wlxpluginTB.Text.Equals( "" ) && (wlxpluginsLB.SelectedIndex >= 0))
			{
				String tmp = ((WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex])._path;
				tmp = sc.PlugPathToPlugins( tmp );
				tmp = sc.AppPathToExe( tmp );

				openFD.InitialDirectory = tmp;
			}

			if (openFD.ShowDialog() == DialogResult.OK)
			{
				FileInfo tmpfile = new FileInfo( openFD.FileName.ToString() );
			
				try
				{
					if (tmpfile.Extension.ToUpper().Equals( ".WLX" ))
					{						
						String tmpext = ((WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex])._exts;
						String tmp2 = tmpfile.FullName;
						tmp2 = sc.PluginsToPlugPath( tmp2 );
						tmp2 = sc.ExePathToAppPath( tmp2 );

						((WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex])._path = tmp2;
						((WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex]).ValName();

						wlxpluginTB.Text = tmp2;
						wlxextensionsTB.Text = tmpext;
						ReloadWLXList();
					}
				}
				catch (Exception ex)
				{
					//
				}
			}
		}

		private void ReloadWLXList()
		{
			List<WLXPlug> lista = new List<WLXPlug>();
			int sel = wlxpluginsLB.SelectedIndex;

			for (int i = 0; i < wlxpluginsLB.Items.Count; i++)
			{
				lista.Add( (WLXPlug) wlxpluginsLB.Items[i] );
			}

			wlxpluginsLB.Items.Clear();

			for (int j = 0; j < lista.Count; j++)
			{
				wlxpluginsLB.Items.Add( new WLXPlug( lista[j]._path, lista[j]._exts ) ); ;
			}
			
			wlxpluginsLB.SelectedIndex = sel;
		}

		private void addwlxBtn_Click( object sender, EventArgs e )
		{
			openFD.Title = this.sc.GetLangText( "WLXPluginDlgTitle" );//"Choose WLX lister plugin";
			openFD.Filter = this.sc.GetLangText( "WLXPluginDlgType" ) + "|*.wlx";
			openFD.FileName = "*.wlx";
			openFD.InitialDirectory = "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}";
						
			if (openFD.ShowDialog() == DialogResult.OK)
			{				
				try
				{
					FileInfo tmpfile = new FileInfo( openFD.FileName.ToString() );

					if (tmpfile.Extension.ToUpper().Equals( ".WLX" ))
					{
						String tmpext = GetWLXExtensions( openFD.FileName.ToString() );

						String tmp2 = tmpfile.FullName;
						tmp2 = sc.PluginsToPlugPath( tmp2 );
						tmp2 = sc.ExePathToAppPath( tmp2 );

						if (tmpext.Equals( "" ) || tmpext.Length == 0)
						{
							tmpext = "*.*";
						}
						
						wlxpluginsLB.Items.Add( new WLXPlug(tmpfile.FullName, tmpext ) );
						wlxpluginsLB.SelectedIndex = wlxpluginsLB.Items.Count - 1;
					}
				}
				catch (Exception ex)
				{
					//
				}
				
				RefreshWLXTab();
			}

		}

		private void removewlxBtn_Click( object sender, EventArgs e )
		{
			int sel = wlxpluginsLB.SelectedIndex;
			edmode = true;
			
			if (wlxpluginsLB.SelectedIndex >= 0)
			{
				wlxpluginsLB.Items.RemoveAt( wlxpluginsLB.SelectedIndex );
			}

			if (wlxpluginsLB.Items.Count > 0)
			{
				if (sel == 0)
				{
					wlxpluginsLB.SelectedIndex = 0;
				}
				else
				{
					if (sel >= wlxpluginsLB.Items.Count)
					{
						wlxpluginsLB.SelectedIndex = wlxpluginsLB.Items.Count - 1;
					}
					else
					{
						wlxpluginsLB.SelectedIndex = sel;
					}
				}
			}
			else
			{
				wlxpluginTB.Text = "";
				wlxextensionsTB.Text = "";
			}

			RefreshWLXTab();
		}

		private void upwlxBtn_Click( object sender, EventArgs e )
		{
			edmode = true;
			int sel = wlxpluginsLB.SelectedIndex;

			if (sel > 0)
			{
				WLXPlug tmp = (WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex - 1];
				wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex - 1] = (WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex];
				wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex] = tmp;
				wlxpluginsLB.SelectedIndex = sel - 1;
			}

		}

		private void downwlxBtn_Click( object sender, EventArgs e )
		{
			edmode = true;
			int sel = wlxpluginsLB.SelectedIndex;

			if (sel < wlxpluginsLB.Items.Count - 1)
			{
				WLXPlug tmp = (WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex + 1];
				wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex + 1] = (WLXPlug) wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex];
				wlxpluginsLB.Items[wlxpluginsLB.SelectedIndex] = tmp;
				wlxpluginsLB.SelectedIndex = sel + 1;
			}

		}

		private void button9_Click( object sender, EventArgs e )
		{
			openFD.Title = this.sc.GetLangText( "ExecutableDlgTitle" );// "Choose compare tool EXE file";
			openFD.Filter = this.sc.GetLangText( "ExecutableDlgType" ) + "|*.exe";
			openFD.FileName = "*.exe";
			openFD.InitialDirectory = "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}";
			
			if (openFD.ShowDialog() == DialogResult.OK)
			{
				try
				{
					FileInfo tmpfile = new FileInfo( openFD.FileName.ToString() );

					if (tmpfile.Extension.ToUpper().Equals( ".EXE" ))
					{						
						String tmp2 = tmpfile.FullName;
						tmp2 = sc.PluginsToPlugPath( tmp2 );
						tmp2 = sc.ExePathToAppPath( tmp2 );												

						comparersLB.Items.Add(new Comparer(tmpfile.FullName, "*.*", "" ) );
						comparersLB.SelectedIndex = comparersLB.Items.Count - 1;
					}
				}
				catch (Exception ex)
				{
					//
				}

				RefreshCompTab();
			}
		}

		private void button8_Click( object sender, EventArgs e )
		{
			int sel = comparersLB.SelectedIndex;
			edmode = true;
			
			if (comparersLB.SelectedIndex >= 0)
			{
				comparersLB.Items.RemoveAt( comparersLB.SelectedIndex );
			}

			if (comparersLB.Items.Count > 0)
			{
				if (sel == 0)
				{
					comparersLB.SelectedIndex = 0;
				}
				else
				{
					if (sel >= comparersLB.Items.Count)
					{
						comparersLB.SelectedIndex = comparersLB.Items.Count - 1;
					}
					else
					{
						comparersLB.SelectedIndex = sel;
					}
				}
			}
			else
			{
				executableTB.Text = "";
				paramTB.Text = "";
				compextensionsTB.Text = "";
			}

			RefreshCompTab();
		}

		private void upcompBtn_Click( object sender, EventArgs e )
		{
			edmode = true;
			int sel = comparersLB.SelectedIndex;

			if (sel > 0)
			{
				Comparer tmp = (Comparer) comparersLB.Items[comparersLB.SelectedIndex - 1];
				comparersLB.Items[comparersLB.SelectedIndex - 1] = (Comparer) comparersLB.Items[comparersLB.SelectedIndex];
				comparersLB.Items[comparersLB.SelectedIndex] = tmp;
				comparersLB.SelectedIndex = sel - 1;
			}
		}

		private void downcompBtn_Click( object sender, EventArgs e )
		{
			edmode = true;
			int sel = comparersLB.SelectedIndex;

			if (sel < comparersLB.Items.Count - 1)
			{
				Comparer tmp = (Comparer) comparersLB.Items[comparersLB.SelectedIndex + 1];
				comparersLB.Items[comparersLB.SelectedIndex + 1] = (Comparer) comparersLB.Items[comparersLB.SelectedIndex];
				comparersLB.Items[comparersLB.SelectedIndex] = tmp;
				comparersLB.SelectedIndex = sel + 1;
			}
		}

		private void button5_Click( object sender, EventArgs e )
		{
			try
			{
				sc.comparers = new List<Comparer>();

				if (comparersLB.SelectedIndex >= 0)
				{
					((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._path = executableTB.Text;
					((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._exts = compextensionsTB.Text;
					((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._param = paramTB.Text;
				}

				for (int i = 0; i < comparersLB.Items.Count; i++)
				{
					String tmpplug = ((Comparer) comparersLB.Items[i])._path;
					String tmpexts = ((Comparer) comparersLB.Items[i])._exts;
					String tmppar = ((Comparer) comparersLB.Items[i])._param;

					sc.comparers.Add( new Comparer( tmpplug, tmpexts, tmppar ) );
				}

				RefreshCompList( comparersLB.SelectedIndex );
			}
			catch (Exception exc)
			{
				//
			}
		}

		private void comparersLB_SelectedIndexChanged( object sender, EventArgs e )
		{
			if (!edmode)
			{
				if ((compind >= 0) && (compind < comparersLB.Items.Count))
				{
					((Comparer) comparersLB.Items[compind])._path = executableTB.Text;
					((Comparer) comparersLB.Items[compind])._exts = compextensionsTB.Text;
					((Comparer) comparersLB.Items[compind])._param = paramTB.Text;
				}
			}
			else
			{
				edmode = false;
			}

			if (comparersLB.SelectedIndex >= 0)
			{
				if (!((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._path.Equals( "\\<TEMPORARY>" ))
				{
					String tmp = ((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._path;
					tmp = sc.PluginsToPlugPath( tmp );
					tmp = sc.ExePathToAppPath( tmp );

					executableTB.Text = tmp;
					compextensionsTB.Text = ((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._exts;
					paramTB.Text = ((Comparer) comparersLB.Items[comparersLB.SelectedIndex])._param;
				}
				else
				{
					executableTB.Text = "";
					compextensionsTB.Text = "";
					paramTB.Text = "";
				}
			}
						
			compind = comparersLB.SelectedIndex;	
		}

		private void comparersLB_DragDrop( object sender, DragEventArgs e )
		{
			string[] files = (string[]) e.Data.GetData( DataFormats.FileDrop );

			foreach (string compdrop in files)
			{
				FileInfo tmpfile = new FileInfo( compdrop );

				if (tmpfile.Extension.ToUpper().Equals( ".EXE" ))
				{
					comparersLB.Items.Add( new Comparer( tmpfile.FullName, "", "" ) );
					comparersLB.SelectedIndex = comparersLB.Items.Count - 1;					
				}
			}

			RefreshCompTab();
		}

		private void showthumbsCB_Click( object sender, EventArgs e )
		{			
			heightLabel.Enabled = showthumbsCB.Checked;
			heightNUD.Enabled = showthumbsCB.Checked;
			showiconsCB.Enabled = showthumbsCB.Checked;
		}

		private void fieldsBtn_Click( object sender, EventArgs e )
		{
			if (this.pluginsLB.Items.Count > 0)
			{
				if (!this.pluginsLB.Items[this.pluginsLB.SelectedIndex].ToString().Equals( "<TEMPORARY>" ))
				{
					Size sajz = new Size( sc.fieldsW, sc.fieldsH );
					FieldsForm ff = new FieldsForm( (Plug) this.pluginsLB.Items[this.pluginsLB.SelectedIndex], this.pluginsLB.SelectedIndex, sajz, this.sc );
					ff.UpdateFields += new MyFieldsEventHandler( UpdateFields );
					ff.ShowDialog( this );
				}
			}
		}

		public void UpdateFields( object sender, String fstr, int indeks, Size sajz )
		{			
			((Plug) this.pluginsLB.Items[this.pluginsLB.SelectedIndex])._fields = fstr;
			sc.fieldsW = sajz.Width;
			sc.fieldsH = sajz.Height;			
		}

		private void defplugpathBtn_Click( object sender, EventArgs e )
		{
			//openFB.Description = "Choose default plugins path";
			openFB.Description = sc.GetLangText("PlugsPathDlg");
			openFB.SelectedPath = "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}";

			if (!defplugpathTB.Text.Equals( "" ))
			{
				String tmp = defplugpathTB.Text;
				tmp = sc.AppPathToExe( tmp );

				openFB.SelectedPath = tmp;
			}

			if (openFB.ShowDialog() == DialogResult.OK)
			{
				String tmp2 = openFB.SelectedPath.ToString();
				ReloadLists( tmp2 );
				tmp2 = sc.ExePathToAppPath( tmp2 );				
				defplugpathTB.Text = tmp2;
								
			}
		}

		private void ReloadLists(String newpath)
		{			
			sc.plugsdir = newpath;
			
			pluginsLB.Items.Clear();
			wlxpluginsLB.Items.Clear();
			comparersLB.Items.Clear();
						
			RefreshList( 0 );
			RefreshWLXList( 0 );
			RefreshCompList( 0 );			
		}

		private void scanBtn_Click( object sender, EventArgs e )
		{
			okBtn.Enabled = false;
			cancelBtn.Enabled = false;
			Debug.WriteLine( "Start scan for plugins" );
			List<String> listwdx = new List<String>();
			List<String> listwlx = new List<String>();

			SearchPlugins( sc.plugsdir, ref listwdx, ref listwlx );

			Debug.WriteLine( "scan WDX" );
			for (int i = 0; i < listwdx.Count; i++)
			{
				//Debug.WriteLine(listwdx[i]);

				if (!WDXExist( listwdx[i], sc.plugins ))
				{
					String tmpdet = GetWDXExtensions( listwdx[i] );

					//Debug.WriteLine( listwdx[i] + "1\t" + tmpdet );

					if (!tmpdet.Equals( "???" ))
					{
						if (tmpdet.Equals( "" ))
						{
							tmpdet = "*.*";
						}

						sc.plugins.Add( new Plug( listwdx[i], tmpdet, "*;" ) );
						//Debug.WriteLine( listwdx[i] + "2\t" + tmpdet );
					}					
				}
			}
			Debug.WriteLine( "scan WLX" );

			//Debug.WriteLine("WLX");
			for (int j = 0; j < listwlx.Count; j++)
			{
				//Debug.WriteLine(listwlx[j]);
				
				if (!WLXExist( listwlx[j], sc.wlxplugins ))
				{					
					String tmpdet2 = GetWLXExtensions(listwlx[j]);

					//Debug.WriteLine( listwlx[j] + "1\t" + tmpdet2 );										

					if (!tmpdet2.Equals( "???" ))
					{
						if (tmpdet2.Equals( "" ))
						{
							tmpdet2 = "*.*";
						}

						sc.wlxplugins.Add( new WLXPlug( listwlx[j], tmpdet2 ) );
						//Debug.WriteLine( listwlx[j] + "2\t" + tmpdet2 );
					}
				}
			}

			Debug.WriteLine( "Refresh list - WDX" );
			RefreshList( 0 );
			Debug.WriteLine( "Refresh list - WLX" );
			RefreshWLXList( 0 );
			Debug.WriteLine( "End scan for plugins" );

			//
			okBtn.Enabled = true;
			cancelBtn.Enabled = true;
		}

		private void SearchPlugins( String sciezka, ref List<String> listwdx, ref List<String> listwlx )
		{
			if (Directory.Exists( sciezka ))
			{
				DirectoryInfo di = new DirectoryInfo( sciezka );

				FileInfo[] fileswdx = di.GetFiles( "*.wdx" );

				foreach (FileInfo plikwdx in fileswdx)
				{
					if (!plikwdx.Name.ToUpper().Equals( "FILEDESC.WDX" ))
					{
						listwdx.Add( plikwdx.FullName );
					}
				}

				FileInfo[] fileswlx = di.GetFiles( "*.wlx" );

				foreach (FileInfo plikwlx in fileswlx)
				{
					listwlx.Add( plikwlx.FullName );
				}

				DirectoryInfo[] dirs = di.GetDirectories();

				foreach (DirectoryInfo dir in dirs)
				{
					SearchPlugins( dir.FullName, ref listwdx, ref listwlx );
				}
			}
		}

		private Boolean WDXExist(String elem, List<Plug> lista)
		{
			for (int i = 0; i < lista.Count; i++)
			{
				if (lista[i]._path.ToUpper().Equals( elem.ToUpper() ))
				{
					return true;
				}					
			}

			return false;
		}

		private Boolean WLXExist(String elem, List<WLXPlug> lista )
		{
			for (int i = 0; i < lista.Count; i++)
			{
				if (lista[i]._path.ToUpper().Equals( elem.ToUpper() ))
				{
					return true;
				}
			}

			return false;
		}

		public event RefreshFormEventHandler RefreshForm
		{
			add
			{
				refreshForm += new RefreshFormEventHandler( value );
			}
			remove
			{
				refreshForm -= new RefreshFormEventHandler( value );
			}
		}

		protected void OnRefreshForm( )
		{
			if (!refreshForm.Equals( null ))
			{
				refreshForm( this);
			}
		}

		private void General_KeyPress( object sender, KeyPressEventArgs e )
		{
			if (e.KeyChar == (char) Keys.Escape)
			{
				this.cancelBtn_Click(this, new EventArgs());
			}
		}

		private void RefreshWDXTab()
		{
			Boolean tmp = false;

			if (pluginsLB.Items.Count > 0)
			{
				tmp = true;
			}

			pluginTB.Enabled = tmp;
			pluginBtn.Enabled = tmp;
			extensionsTB.Enabled = tmp;
			fieldsBtn.Enabled = tmp;
		}

		private void RefreshWLXTab()
		{
			Boolean tmp = false;

			if (wlxpluginsLB.Items.Count > 0)
			{
				tmp = true;
			}

			wlxpluginTB.Enabled = tmp;
			wlxpluginBtn.Enabled = tmp;
			wlxextensionsTB.Enabled = tmp;
		}

		private void RefreshCompTab()
		{
			Boolean tmp = false;

			if (comparersLB.Items.Count > 0)
			{
				tmp = true;
			}			

			executableTB.Enabled = tmp;
			executableBtn.Enabled = tmp;
			paramTB.Enabled = tmp;
			compextensionsTB.Enabled = tmp;
		}

		private void nameBtn_Click( object sender, EventArgs e )
		{
			ContextMenu cms = new ContextMenu();
			MenuItem mi = new MenuItem();

			if (((Button) sender).Name.ToString().Equals( "singlenameBtn" ))
			{
				cms = new ContextMenu();

				mi = new MenuItem();
				mi.Text = "%fs\t\t" + this.sc.GetLangText( "SFileMenuShort" );//short file name";
				mi.Tag = (Int32) 1;
				mi.Click += new EventHandler( mi_Click );
				cms.MenuItems.Add( mi );				
				
				mi = new MenuItem();
				mi.Text = "%ff\t\t" + this.sc.GetLangText( "SFileMenuFull" );//full file name";
				mi.Tag = (Int32) 2;
				mi.Click += new EventHandler( mi_Click );
				cms.MenuItems.Add( mi );
								
				mi = new MenuItem();
				mi.Text = "-";
				mi.Tag = (String) "separator";
				cms.MenuItems.Add( mi );

				mi = new MenuItem();
				mi.Text = "%df\t\t" + this.sc.GetLangText( "SFileMenuDate" );//date && time (formatted)";
				mi.Tag = (Int32) 3;
				mi.Click += new EventHandler( mi_Click );
				cms.MenuItems.Add( mi );			
			}
			else
			{
				if (((Button) sender).Name.ToString().Equals( "doublenameBtn" ))
				{
					cms = new ContextMenu();

					mi = new MenuItem();
					mi.Text = "%f1s\t\t" + this.sc.GetLangText( "DFileMenuFirstShort" );//short first file name";
					mi.Tag = (Int32) 4;
					mi.Click += new EventHandler( mi_Click );
					cms.MenuItems.Add( mi );

					mi = new MenuItem();
					mi.Text = "%f1f\t\t" + this.sc.GetLangText( "DFileMenuFirstFull" );//full first file name";
					mi.Tag = (Int32) 5;
					mi.Click += new EventHandler( mi_Click );
					cms.MenuItems.Add( mi );
										
					mi = new MenuItem();
					mi.Text = "-";
					mi.Tag = (String) "separator";
					cms.MenuItems.Add( mi );

					mi = new MenuItem();
					mi.Text = "%f2s\t\t" + this.sc.GetLangText( "DFileMenuSecondShort" );//short second file name";
					mi.Tag = (Int32) 6;
					mi.Click += new EventHandler( mi_Click );
					cms.MenuItems.Add( mi );

					mi = new MenuItem();
					mi.Text = "%f2f\t\t" + this.sc.GetLangText( "DFileMenuSecondFull" );//full second file name";
					mi.Tag = (Int32) 7;
					mi.Click += new EventHandler( mi_Click );
					cms.MenuItems.Add( mi );
										
					mi = new MenuItem();
					mi.Text = "-";
					mi.Tag = (String) "separator";
					cms.MenuItems.Add( mi );

					mi = new MenuItem();
					mi.Text = "%df\t\t" + this.sc.GetLangText( "DFileMenuDate" );//date && time (formatted)";
					mi.Tag = (Int32) 8;
					mi.Click += new EventHandler( mi_Click );
					cms.MenuItems.Add( mi );
				}
				else
				{
					if (((Button) sender).Name.ToString().Equals( "singleimageBtn" ))
					{
						cms = new ContextMenu();

						mi = new MenuItem();
						mi.Text = "%fs\t\t" + this.sc.GetLangText( "ImgSFileMenuShort" );//short file name";
						mi.Tag = (Int32) 9;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "%ff\t\t" + this.sc.GetLangText( "ImgSFileMenuFull" );//full file name";
						mi.Tag = (Int32) 10;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "%fp\t\t" + this.sc.GetLangText( "ImgSFileMenuPath" );//file path";
						mi.Tag = (Int32) 21;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "-";
						mi.Tag = (String) "separator";
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "%df\t\t" + this.sc.GetLangText( "ImgSFileMenuDate" );//date && time (formatted)";
						mi.Tag = (Int32) 11;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "-";
						mi.Tag = (String) "separator";
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "\\n\t" + this.sc.GetLangText( "ImgSFileMenuLine" );//new line";
						mi.Tag = (Int32) 17;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "\\t\t" + this.sc.GetLangText( "ImgSFileMenuTab" );//tabulation";
						mi.Tag = (Int32) 18;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );
					}
					else
					{
						cms = new ContextMenu();

						mi = new MenuItem();
						mi.Text = "%f1s\t\t" + this.sc.GetLangText( "ImgDFileMenuFirstShort" );//short first file name";
						mi.Tag = (Int32) 12;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "%f1f\t\t" + this.sc.GetLangText( "ImgDFileMenuFirstFull" );//full first file name";
						mi.Tag = (Int32) 13;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "%f1p\t\t" + this.sc.GetLangText( "ImgDFileMenuFirstPath" );//first file path";
						mi.Tag = (Int32) 22;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "-";
						mi.Tag = (String) "separator";
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "%f2s\t\t" + this.sc.GetLangText( "ImgDFileMenuSecondShort" );//short second file name";
						mi.Tag = (Int32) 14;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "%f2f\t\t" + this.sc.GetLangText( "ImgDFileMenuSecondFull" );//full second file name";
						mi.Tag = (Int32) 15;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "%f2p\t\t" + this.sc.GetLangText( "ImgDFileMenuSecondPath" );//second file path";
						mi.Tag = (Int32) 23;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "-";
						mi.Tag = (String) "separator";
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "%df\t\t" + this.sc.GetLangText( "ImgDFileMenuDate" );//date && time (formatted)";
						mi.Tag = (Int32) 16;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "-";
						mi.Tag = (String) "separator";
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "\\n\t" + this.sc.GetLangText( "ImgDFileMenuLine" );//new line";
						mi.Tag = (Int32) 19;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );

						mi = new MenuItem();
						mi.Text = "\\t\t" + this.sc.GetLangText( "ImgDFileMenuTab" );//tabulation";
						mi.Tag = (Int32) 20;
						mi.Click += new EventHandler( mi_Click );
						cms.MenuItems.Add( mi );
					}
				}
			}

			((Button) sender).ContextMenu = cms;
			cms.Show( (Button) sender, new Point( 0, ((Button) sender).Height ) );
		}

		void mi_Click( object sender, EventArgs e )
		{
			try
			{
				Int32 itemnr = Int32.Parse( ((MenuItem) sender).Tag.ToString() );

				switch (itemnr)
				{
					case 1:
						singlenameTB.Text = singlenameTB.Text + "%fs";
						break;
					case 2:
						singlenameTB.Text = singlenameTB.Text + "%ff";
						break;
					case 3:
						singlenameTB.Text = singlenameTB.Text + "%df(yyyMMdd)";
						break;
					case 4:
						doublenameTB.Text = doublenameTB.Text + "%f1s";
						break;
					case 5:
						doublenameTB.Text = doublenameTB.Text + "%f1f";
						break;
					case 6:
						doublenameTB.Text = doublenameTB.Text + "%f2s";
						break;
					case 7:
						doublenameTB.Text = doublenameTB.Text + "%f2f";
						break;
					case 8:
						doublenameTB.Text = doublenameTB.Text + "%df(yyyMMdd)";
						break;
					case 9:
						singleimageTB.Text = singleimageTB.Text + "%fs";
						break;
					case 10:
						singleimageTB.Text = singleimageTB.Text + "%ff";
						break;
					case 11:
						singleimageTB.Text = singleimageTB.Text + "%df(yyyMMdd)";
						break;
					case 12:
						doubleimageTB.Text = doubleimageTB.Text + "%f1s";
						break;
					case 13:
						doubleimageTB.Text = doubleimageTB.Text + "%f1f";
						break;
					case 14:
						doubleimageTB.Text = doubleimageTB.Text + "%f2s";
						break;
					case 15:
						doubleimageTB.Text = doubleimageTB.Text + "%f2f";
						break;
					case 16:
						doubleimageTB.Text = doubleimageTB.Text + "%df(yyyMMdd)";
						break;
					case 17:
						singleimageTB.Text = singleimageTB.Text + "\\n";
						break;
					case 18:
						singleimageTB.Text = singleimageTB.Text + "\\t";
						break;
					case 19:
						doubleimageTB.Text = doubleimageTB.Text + "\\n";
						break;
					case 20:
						doubleimageTB.Text = doubleimageTB.Text + "\\t";
						break;					
					case 21:
						singleimageTB.Text = singleimageTB.Text + "%fp";
						break;
					case 22:
						doubleimageTB.Text = doubleimageTB.Text + "%f1p";
						break;
					case 23:
						doubleimageTB.Text = doubleimageTB.Text + "%f2p";
						break;
					default:
						break;
				}				
			}
			catch (Exception exc)
			{
				MessageBox.Show( exc.ToString() );
			}
		}

		private void shownameBtn_Click( object sender, EventArgs e )
		{
			NamesForm nf = new NamesForm( singlenameTB.Text, doublenameTB.Text, this.sc );
			nf.ShowDialog( this );
		}

		private void showimageBtn_Click( object sender, EventArgs e )
		{
			ImagesForm imgf = new ImagesForm( singleimageTB.Text, doubleimageTB.Text, (Int32) imgwidthNUD.Value, (Int32) imgheightNUD.Value, imagefontFontD.Font, imgcolourCD.Color, this.sc );
			imgf.ShowDialog(this);			
		}

		private void fontBtn_Click( object sender, EventArgs e )
		{
			if (imagefontFontD.ShowDialog() == DialogResult.OK)
			{
				imagefontLabel.Text = imagefontFontD.Font.Name.ToString() + ", " + imagefontFontD.Font.Style.ToString() + ", " + imagefontFontD.Font.Size.ToString();				
			}
		}

		private void showimgheadCB_Click( object sender, EventArgs e )
		{
			singleimageLabel.Enabled = showimgheadCB.Checked;
			doubleimageLabel.Enabled = showimgheadCB.Checked;
			singleimageTB.Enabled = showimgheadCB.Checked;
			doubleimageTB.Enabled = showimgheadCB.Checked;
			singleimageBtn.Enabled = showimgheadCB.Checked;
			doubleimageBtn.Enabled = showimgheadCB.Checked;
			showimageBtn.Enabled = showimgheadCB.Checked;
			heightimageLabel.Enabled = showimgheadCB.Checked;
			imgheightNUD.Enabled = showimgheadCB.Checked;
			widthimageLabel.Enabled = showimgheadCB.Checked;
			imgwidthNUD.Enabled = showimgheadCB.Checked;
			imgfontLabel.Enabled = showimgheadCB.Checked;
			imagefontLabel.Enabled = showimgheadCB.Checked;
			imagefontBtn.Enabled = showimgheadCB.Checked;
			imgcolourLabel.Enabled = showimgheadCB.Checked;
			imagecolourLabel.Enabled = showimgheadCB.Checked;
			imagecolourBtn.Enabled = showimgheadCB.Checked;
		}

		private void imagecolourBtn_Click( object sender, EventArgs e )
		{
			if (imgcolourCD.ShowDialog() == DialogResult.OK)
			{
				//imagecolourLabel.Text = imgcolourCD.Color.Name.ToString();
				imagecolourLabel.Text = "RGB ( " + imgcolourCD.Color.R.ToString() + ", " + imgcolourCD.Color.G.ToString() + ", " + imgcolourCD.Color.B.ToString() + " )";
				
				imagecolourLabel.ForeColor = imgcolourCD.Color;
			}
		}

		private void datefrmtLLabel_LinkClicked( object sender, LinkLabelLinkClickedEventArgs e )
		{
			String tmp = Application.ExecutablePath;
			tmp = tmp.Substring( 0, tmp.LastIndexOf( "\\" ) + 1 );
			tmp = tmp + "datafrmt.rtf";

			if (File.Exists( tmp ))
			{
				Process proc = new Process();
				proc.StartInfo.UseShellExecute = true;
				proc.StartInfo.FileName = tmp;
				proc.Start();
			}
			else
			{
				String tmperr = "Help file cannot be opened!";
				tmperr = tmperr + Environment.NewLine + Environment.NewLine;
				tmperr = tmperr + "File:" + Environment.NewLine + Environment.NewLine;
				tmperr = tmperr + tmp + Environment.NewLine + Environment.NewLine + "is missing.";
				MessageBox.Show( this, tmperr, "Help file error", MessageBoxButtons.OK, MessageBoxIcon.Error );
			}
		}

		private void CheckLangs()
		{
			this._langs = new List<LangElem>();

			//reading lang files

			String tmplang = "%APP_PATH%\\langs\\";
			tmplang = sc.AppPathToExe( tmplang );

			this._langs.Add( new LangElem( "$" ));

			if (Directory.Exists( tmplang ))
			{
				//proceed
				DirectoryInfo di = new DirectoryInfo( tmplang );
				FileInfo[] langfiles = di.GetFiles( "*.ini" );

				for (int i = 0; i < langfiles.Length; i++)
				{
					this._langs.Add( new LangElem( tmplang + langfiles[i].ToString() ) );
				}
			}
		}

		private void SetFormLang()
		{
			//TODO: set controls texts
			//= setc.GetLangText( "" );

			this.Text = sc.GetLangText( "OptTitle" );
			okBtn.Text = sc.GetLangText( "OKBtn" );
			cancelBtn.Text = sc.GetLangText( "CancelBtn" );

			
			//general tab
			tabPage1.Text = sc.GetLangText( "GenTitle" );

			//general tab - Application window
			appGB.Text = sc.GetLangText( "AppWinGr" );
			sizeCB.Text = sc.GetLangText( "AppWinSize" );
			defaultBtn.Text = sc.GetLangText( "AppWinResetBtn" );
			ontopCB.Text = sc.GetLangText( "AppWinAlwaysTop" );
			closecmpCB.Text = sc.GetLangText( "AppWinCloseCompare" );
			stickmodeCB.Text = sc.GetLangText( "AppWinStickMode" );

			//general tab - Language
			langGB.Text = sc.GetLangText( "LangTitle" );
			langautorLabel.Text = sc.GetLangText( "LangAuthor" );
			langversionlabel.Text = sc.GetLangText( "LangVersion" );
			langfileLabel.Text = sc.GetLangText( "LangFile" );

			//general tab - Thumbnails view
			thumbGB.Text = sc.GetLangText( "ThViewGr" );
			showthumbsCB.Text = sc.GetLangText( "ThViewShow" );
			heightLabel.Text = sc.GetLangText( "ThViewHeight" );
			showiconsCB.Text = sc.GetLangText( "ThViewIcons" );

			//general tab - Visibility
			visibilityGB.Text = sc.GetLangText( "VisGr" );
			showdifferentCB.Text = sc.GetLangText( "VisOnlyDiff" );
			showemptyCB.Text = sc.GetLangText( "VisEmpty" );
			wordwrapCB.Text = sc.GetLangText( "VisWrap" );

			//general tab - Plugins
			genplugGB.Text = sc.GetLangText( "PlugsDefGr" );
			defplugpathLabel.Text = sc.GetLangText( "PlugsDefPath" );
			scanBtn.Text = sc.GetLangText( "PlugsScanBtn" );
			


			//Plugins WDX tab
			tabPage2.Text = sc.GetLangText( "WDXTitle" );
			upBtn.Text = sc.GetLangText( "WDXUpBtn" );
			downBtn.Text = sc.GetLangText( "WDXDownBtn" );
			fieldsBtn.Text = sc.GetLangText( "SetFieldsBtn" );
			
			//Plugins wdx tab - plugin
			wdxplugGB.Text = sc.GetLangText( "WDXPluginGr" );

			//Plugins wdx tab - extensions
			wdxextsGB.Text = sc.GetLangText( "WDXExtensionsGr" );


			//Plugins WLX tab
			tabPage4.Text = sc.GetLangText( "WLXTitle" );
			upwlxBtn.Text = sc.GetLangText( "WLXUpBtn" );
			downwlxBtn.Text = sc.GetLangText( "WLXDownBtn" );			

			//Plugins wlx tab - plugin
			wlxplugGB.Text = sc.GetLangText( "WLXPluginGr" );

			//Plugins wlx tab - extensions
			wlxextsGB.Text = sc.GetLangText( "WLXExtensionsGr" );



			//Comparers tab
			tabPage6.Text = sc.GetLangText( "CmpTitle" );
			upcmpBtn.Text = sc.GetLangText( "CmpUpBtn" );
			downcmpBtn.Text = sc.GetLangText( "CmpDownBtn" );

			//Comparers tab - executable
			cmpexeGB.Text = sc.GetLangText( "ExecutableGr" );
			cmpparamsLabel.Text = sc.GetLangText( "Parameters" );
			cmpparinfoLabel.Text = sc.GetLangText( "ParametersHint" );

			//Comparers tab - extensions
			cmpextsGB.Text = sc.GetLangText( "CmpExtensionsGr" );



			//Export tab
			tabPage5.Text = sc.GetLangText( "ExportTitle" );

			//Export tab - default files names
			filesnamesGB.Text = sc.GetLangText( "FilenamesGr" );
			singleLabel.Text = sc.GetLangText( "SingleFile" );
			doubleLabel.Text = sc.GetLangText( "DoubleFile" );
			shownameBtn.Text = sc.GetLangText( "ShowBtn" );

			//Export tab - CSV
			csvGB.Text = sc.GetLangText( "CSVGr" );
			separatorLabel.Text = sc.GetLangText( "CSVSeparator" );
			quotesCB.Text = sc.GetLangText( "CSVSurround" );

			//Export tab - Image
			imageGB.Text = sc.GetLangText( "ImageGr" );
			showimgheadCB.Text = sc.GetLangText( "ImgDrawHead" );
			singleimageLabel.Text = sc.GetLangText( "ImgSFile" );
			doubleimageLabel.Text = sc.GetLangText( "ImgDFile" );
			showimageBtn.Text = sc.GetLangText( "ImgShowBtn" );

			widthimageLabel.Text = sc.GetLangText( "ImgGrWidth" );
			heightimageLabel.Text = sc.GetLangText( "ImgGrHeight" );

			imgfontLabel.Text = sc.GetLangText( "ImgGrFont" );
			imgcolourLabel.Text = sc.GetLangText( "ImgGrColour" );

			imagefontBtn.Text = sc.GetLangText( "ImgGrFontChangeBtn" );
			imagecolourBtn.Text = sc.GetLangText( "ImgGrColourChangeBtn" );

			//Export tab - ...
			runappCB.Text = sc.GetLangText( "OpenAfterExp" );
			datefrmtLLabel.Text = sc.GetLangText( "DateFormatInfo" );



			//Info tab
			tabPage3.Text = sc.GetLangText( "InfoTitle" );

			String tmpinfo = sc.GetLangText( "InfoContent" );
			tmpinfo = tmpinfo.Replace( "\\n", "\r\n" );

			infoRTB.Text = tmpinfo;

		}

		private void FillLangsCombo(String langfile)
		{
			foreach (LangElem lng in this._langs)
			{
				langCB.Items.Add( lng );

				if (lng._file.Equals( langfile ))
				{
					langCB.Text = lng._name;
				}
			}
		}

		private void langCB_SelectedIndexChanged( object sender, EventArgs e )
		{
			LangElem lem = (LangElem) langCB.SelectedItem;
			authorLabel.Text = lem._author;
			versionLabel.Text = lem._version;

			if (!lem._file.Equals( "$" ))
			{
				fileLabel.Text = lem._file;
			}
			else
			{
				fileLabel.Text = "";
			}
		}
	}
}
