﻿/*
 * HISTORY:
 * 20081225 - changed parameters parsing
 * 20081225 - added "-a" switch for autoomatic plugins dir scanning
 * 20081226 - fixed x64 issue;
 * 20081226 - added "-d" switch for debug mode (empty fields or plugins are not shown by default);
 * ----------------------------------------------------------
 * 20090105 - fixed ft_datetime a little bit
 * 20090105 - added ft_string vorrection procedure
 * ----------------------------------------------------------
 * 20090226 - changed internal plugins handling to wdx_proxy.dll
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;
using System.Diagnostics;

namespace ComparerCmd
{
	class Program
	{
		static SettingsClass setc;
		static Boolean givehtml;
		static String myini;
		static String mycss;
		static String myhtml;
		static TextWriter tw;
		static TextWriter twhtml;
		static List<String> lista;
		static String firstfile;
		static Boolean scandirs;
		static List<String> listwdx;
		static Boolean debugmode;

		static void Main( string[] args )
		{
			givehtml = false;
			myini = Assembly.GetExecutingAssembly().Location;
			myini = myini.Remove( myini.LastIndexOf( "\\" ) + 1 );
			mycss = myini + "output.css";
			myhtml = myini + "output.html";
			myini = myini + "comparer.ini";
			lista = new List<String>();
			firstfile = "";
			scandirs = false;
			debugmode = false;

			if (args.Length > 0)
			{
				if (args.Length == 1)
				{
					if (args[0].ToUpper().Equals( "-VER" ))
					{
						ShowVerInfo();
					}
					else
					{
						firstfile = args[0];
						ProcessFile( firstfile );
					}
				}
				else
				{
					for (int i = 0; i < args.Length; i++)
					{
						lista.Add( args[i] );
					}

					givehtml = CheckParam( ref lista, "-h" );
					CheckParam( ref lista, "-c" );
					CheckParam( ref lista, "-a" );
					CheckParam( ref lista, "-d" );

					if (lista.Count > 0)
					{
						if (lista.Count == 1)
						{
							firstfile = lista[0];
							ProcessFile( firstfile );
						}
					}
					else
					{
						PrintInfo();
					}
				}
			}
			else
			{
				PrintInfo();
			}

			//Console.ReadKey();
		}

		static void ShowVerInfo()
		{
			Console.WriteLine( "" );
			Console.WriteLine( "--ComparerCmd info--" );
			Console.WriteLine( "" );
			
			Assembly asm = Assembly.GetExecutingAssembly();
			Console.WriteLine( "Version: " + asm.GetName().Version.ToString());
			
			object[] attributes = asm.GetCustomAttributes (typeof(AssemblyTitleAttribute), false);
			if (attributes.Length == 1)
			{
				Console.WriteLine ("Title:\t" + ((AssemblyTitleAttribute) attributes[0]).Title);				
			}

			attributes = asm.GetCustomAttributes( typeof( AssemblyDescriptionAttribute ), false );
			if (attributes.Length == 1)
			{
				Console.WriteLine("Description:\t" + ((AssemblyDescriptionAttribute) attributes[0]).Description );
			}

			attributes = asm.GetCustomAttributes( typeof( AssemblyCompanyAttribute ), false );
			if (attributes.Length == 1)
			{
				Console.WriteLine( "Company:\t" + ((AssemblyCompanyAttribute) attributes[0]).Company );
			}

			attributes = asm.GetCustomAttributes( typeof( AssemblyProductAttribute ), false );
			if (attributes.Length == 1)
			{
				Console.WriteLine( "Product:\t" + ((AssemblyProductAttribute) attributes[0]).Product);
			}

			attributes = asm.GetCustomAttributes( typeof( AssemblyCopyrightAttribute ), false );
			if (attributes.Length == 1)
			{
				Console.WriteLine( "Copyright:\t" + ((AssemblyCopyrightAttribute) attributes[0]).Copyright );
			}

			Console.WriteLine( "Special thanks:\t" + "Franck8244 (for a work with wdx_proxy.dll)" );
			
			
		}

		static void ProcessFile(String filename)
		{			
			setc = new SettingsClass( myini );
			setc.ReadGeneral();
			GetSingleFileInfo( filename );
		}

		static void GetSingleFileInfo( String filename )
		{
			if (givehtml)
			{
				GenerateCSS();
				GenerateHTMLHead(filename);
			}

			if (!File.Exists( filename ))
			{
				if (givehtml)
				{
					twhtml.WriteLine( "<tr>\r\n<th class=\"nofile\" colspan=\"2\">File does not exist!</th>\r\n</tr>" );

					twhtml.WriteLine( "</table>\r\n</div>\r\n" );

					String tmpfoot = GetFromFile( "footer.html" );

					if (!tmpfoot.Equals( "" ))
					{
						tmpfoot = tmpfoot.Replace( "%f", GetFilePath( filename ) );
						tmpfoot = tmpfoot.Replace( "%p", GetPath( filename ) );
						twhtml.WriteLine( tmpfoot );
					}

					twhtml.WriteLine( "</body>\r\n</html>" );
					twhtml.Close();
				}
				else
				{
					Console.WriteLine( "File \"" + filename + "\" does not exist!" );
				}
			}
			else
			{

				CreateGeneralInfo( filename );

				String ext = filename;
				ext = ext.Remove( 0, ext.LastIndexOf( "." ) );

				setc.ReadPlugins( ext );

				if (scandirs)
				{
					listwdx = new List<String>();
					SearchPlugins( setc.plugsdir, ref listwdx );

					for (int i = 0; i < listwdx.Count; i++)
					{
						//Debug.WriteLine(listwdx[i]);

						if (!WDXExist( listwdx[i], setc.plugins ))
						{								
							setc.plugins.Add( new Plug( listwdx[i], "*.*", "*;" ) );
							//Debug.WriteLine( listwdx[i] + "2\t" + tmpdet );
						}
					}
				}

				CreatePluginsInfo( filename );

				if (givehtml)
				{
					twhtml.WriteLine( "</table>\r\n</div>\r\n" );

					String tmpfoot = GetFromFile( "footer.html" );

					if (!tmpfoot.Equals( "" ))
					{
						tmpfoot = tmpfoot.Replace( "%f", GetFilePath( filename ) );
						tmpfoot = tmpfoot.Replace( "%p", GetPath( filename ) );
						twhtml.WriteLine( tmpfoot );
					}

					twhtml.WriteLine( "</body>\r\n</html>" );
					twhtml.Close();
				}
			}
		}

		static void CreateGeneralInfo( String firstFile )
		{
			if (!givehtml)
			{
				Console.WriteLine( "" );
				Assembly asm = Assembly.GetExecutingAssembly();
				Console.WriteLine( "ComparerCmd " + asm.GetName().Version.ToString() );
				Console.WriteLine( "");	

				FileInfo file1 = new FileInfo( firstFile );
				Console.WriteLine( "========================================" );
				Console.WriteLine( "General");
				Console.WriteLine( "========================================" );
				Console.WriteLine( "Name:\t" + file1.Name.ToString() );				
				Console.WriteLine( "Path:\t" + file1.DirectoryName.ToString() );
				Console.WriteLine( "Size:\t" + file1.Length.ToString() + " b" );
				Console.WriteLine( "Attributes:\t" + file1.Attributes.ToString() );
				Console.WriteLine( "Created:\t" + file1.CreationTime.ToString() );
				Console.WriteLine( "Last Access:\t" + file1.LastAccessTime.ToString() );
				Console.WriteLine( "Last Write:\t" + file1.LastWriteTime.ToString() );
				Console.WriteLine( "========================================" );
				Console.WriteLine( "" );
			}
			else
			{
				FileInfo file1 = new FileInfo( firstFile );
				Assembly asm = Assembly.GetExecutingAssembly();
				
				twhtml.WriteLine( "<tr>\r\n<th colspan=\"2\">General</th>\r\n</tr>" );
				twhtml.WriteLine("<tr><td>Name</td><td>"+ file1.Name.ToString() +"</td></tr>");
				twhtml.WriteLine("<tr><td>Path</td><td>"+ file1.DirectoryName.ToString() +"</td></tr>");
				twhtml.WriteLine("<tr><td>Size</td><td>" + file1.Length.ToString() + "b</td></tr>" );
				twhtml.WriteLine("<tr><td>Attributes</td><td>"+ file1.Attributes.ToString() +"</td></tr>");
				twhtml.WriteLine("<tr><td>Created</td><td>"+ file1.CreationTime.ToString() +"</td></tr>");
				twhtml.WriteLine("<tr><td>Last Access</td><td>"+ file1.LastAccessTime.ToString() +"</td></tr>");
				twhtml.WriteLine("<tr><td>Last Write</td><td>"+ file1.LastWriteTime.ToString() +"</td></tr>");

				//twhtml.WriteLine("


				//twhtml.Close();
			}
		}

		static void CreatePluginsInfo( String firstFile )
		{			
			if (setc.plugins.Count > 0)
			{

				PluginProxy plug = new PluginProxy();
				plug.Load( );

				String tmpplugs = "";
				for (int i = 0; i < setc.plugins.Count; i++)
				{
					tmpplugs = tmpplugs + setc.plugins[i]._path + "?";
				}
				
				tmpplugs = tmpplugs.TrimEnd( '?' );
				plug.InitializePlugins( tmpplugs );

				for (int i = 0; i < setc.plugins.Count; i++)
				{
					String tmp1 = "";
					String tmpflds = "*";
					List<FieldInfo> fieldslist = new List<FieldInfo>();
					String tmpfieldstr = plug.GetFields( i );
					String[] tmpfields = tmpfieldstr.Split( ';' );

					for (int j = 0; j < tmpfields.Length; j++)
					{
						tmpflds = setc.plugins[i]._fields;

						if (tmpflds.Contains( "*" ) || tmpflds.Contains( ";" + j.ToString() + ";" ))
						{
							tmp1 = plug.GetValueStr( tmpfields[j], firstFile, i );
							tmp1 = tmp1.TrimEnd( Convert.ToChar( (Int32) 0 ) );

							if (!tmp1.Equals( "" ))
							{
								fieldslist.Add( new FieldInfo( tmpfields[j], tmp1 ) );		
							}
						}
					}
					

					if (fieldslist.Count == 0)
					{
						if (debugmode)
						{
							if (!givehtml)
							{
								Console.WriteLine( "========================================" );
								Console.WriteLine( "Plugin:\t" + setc.plugins[i]._name + "<NO FIELDS>");
								Console.WriteLine( "----------------------------------------" );
								//plugin fields
								Console.WriteLine( "" );
							}
							else
							{
								twhtml.WriteLine( "<tr>\r\n<th class=\"nofields\" colspan=\"2\">" + setc.plugins[i]._name + " &lt;NO FIELDS&gt;</th>\r\n</tr>" );
							}
						}
					}
					else
					{
						if (!givehtml)
						{
							Console.WriteLine( "========================================" );
							Console.WriteLine( "Plugin:\t" + setc.plugins[i]._name );
							Console.WriteLine( "----------------------------------------" );

							for (int k = 0; k < fieldslist.Count; k++)
							{
								Console.WriteLine( fieldslist[k]._fname + ":\t" + fieldslist[k]._fvalue );
							}
							
							Console.WriteLine( "========================================" );
							Console.WriteLine( "" );
							
						}
						else
						{
							twhtml.WriteLine( "<tr>\r\n<th colspan=\"2\">" + setc.plugins[i]._name + "</th>\r\n</tr>" );

							for (int l = 0; l < fieldslist.Count; l++)
							{
								twhtml.WriteLine( "<tr><td>" + fieldslist[l]._fname + "</td><td>" + fieldslist[l]._fvalue + "</td></tr>" );
							}
						}
					}					
				}
				plug.Free();				
			}
			else
			{
				if (debugmode)
				{
					if (!givehtml)
					{
						Console.WriteLine( "<NO PLUGINS>" );
					}
					else
					{
						twhtml.WriteLine( "<tr>\r\n<th colspan=\"2\">&lt;NO PLUGINS&gt;</th>\r\n</tr>" );
					}
				}
			}
		}

		static void GenerateCSS()
		{
			if (!File.Exists( mycss ))
			{
				//create CSS
				tw = new StreamWriter( mycss );

				tw.WriteLine( "body\r\n{\r\nbackground-color: White;\r\nmargin: 0;\r\npadding: 0;text-align: center;\r\n}" );
				tw.WriteLine( "#main\r\n{\r\ntop: 0;\r\nright: 0;\r\nbottom: 0;\r\nleft: 0;\r\nwidth: 50%;\r\nmargin: auto;\r\n}" );
				tw.WriteLine( "h1\r\n{\r\nfont-weight:bold;\r\nfont-size:x-large;\r\ncolor: rgb(62,196,228);\r\n}" );
				tw.WriteLine( ".tablemain\r\n{\r\ntext-align:left;\r\nfont-size: small;\r\ncolor:Black;\r\nbackground:rgb(239,239,239);" );
				tw.WriteLine( "border: solid 1px rgb(62,196,228);\r\nempty-cells:show;\r\nborder-spacing:0px;\r\nborder-collapse:collapse;\r\n}" );
				tw.WriteLine( "th\r\n{\r\ntext-align:left;\r\nfont-weight:bold;\r\ncolor:Black;\r\nbackground: rgb(185,232,244);" );
				tw.WriteLine( "border: solid 1px rgb(62,196,228);\r\npadding: 2px 2px 2px 2px;\r\n}" );
				tw.WriteLine( "td\r\n{\r\nborder: solid 1px rgb(62, 196, 228);\r\npadding: 5px 5px 5px 5px;\r\n}" );
				tw.WriteLine( ".nofile\r\n{\r\nborder: solid 1px rgb(255,0,0);\r\npadding: 5px 5px 5px 5px;" );
				tw.WriteLine( "background:rgb(255,255,0);\r\ncolor:Black;\r\ntext-align:center;\r\n}" );
				tw.WriteLine( ".nofields\r\n{\r\nborder: solid 1px rgb(62, 196, 228);\r\npadding: 5px 5px 5px 5px;\r\n}" );
				tw.Close();
			}
		}
		
		static void GenerateHTMLHead(String filename)
		{
			if (File.Exists( myhtml ))
			{
				File.Delete( myhtml );
			}

			twhtml = new StreamWriter( myhtml );

			twhtml.WriteLine( "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">" );
			twhtml.WriteLine( "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">" );
			twhtml.WriteLine( "<head>" );
			twhtml.WriteLine( "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />" );
			twhtml.WriteLine( "<meta name=\"keywords\" content=\"ComparerCmd,fenixproductions,\" />" );
			twhtml.WriteLine( "<meta name=\"description\" content=\"ComparerCmd - file's properties information.\" />" );
			twhtml.WriteLine( "<link rel=\"stylesheet\" type=\"text/css\" href=\"output.css\" />" );
			twhtml.WriteLine( "<title>ComparerCmd - " + filename + "</title>" );

			twhtml.WriteLine( "</head>\r\n<body>\r\n<div id=\"main\">" );
			//twhtml.WriteLine( "<h1>" + filename + "</h1>" );

			String tmphead = GetFromFile( "header.html" );

			if (tmphead.Equals( "" ))
			{
				twhtml.WriteLine( "<h1>" + filename + "</h1>" );
			}
			else
			{
				tmphead = tmphead.Replace( "%f", GetFilePath(filename ));
				tmphead = tmphead.Replace( "%p", GetPath(filename) );
				twhtml.WriteLine( tmphead );
			}

			twhtml.WriteLine( "<table class=\"tablemain\" width=\"100%\">" );
		}

		static String GetPath(String filename)
		{
			filename = filename.Substring( 0, filename.LastIndexOf( '\\' ) + 1 );

			return filename;
		}

		static String GetFilePath( String filename )
		{
			filename = filename.Substring( filename.LastIndexOf( '\\' ) + 1, filename.Length - filename.LastIndexOf( '\\' ) -1);

			return filename;
		}

		static String GetFromFile(String extfile)
		{
			String result = "";

			String myhead = Assembly.GetExecutingAssembly().Location;
			myhead = myhead.Remove( myini.LastIndexOf( "\\" ) + 1 );
			myhead = myhead + extfile;

			if (File.Exists( myhead ))
			{
				TextReader tr = new StreamReader( myhead );

				String line = "";

				while ((line = tr.ReadLine()) != null)
				{
					result = result + line + "\r\n";
				}

				tr.Close();
			}			

			return result;
		}

		static Boolean CheckParam( ref List<String> lista, String par )
		{
			for (int i = 0; i < lista.Count; i++)
			{
				if (lista[i].ToUpper().Equals( par.ToUpper() ))
				{
					if (par.ToUpper().Equals( "-H" ))
					{
						givehtml = true;
						lista.RemoveAt( i );
						return true;
					}

					if (par.ToUpper().Equals( "-C" ))
					{
						if ((i + 1) < lista.Count)
						{
							myini = lista[i + 1];
							lista.RemoveAt( i + 1 );
							lista.RemoveAt( i );
							return true;
						}
						else
						{
							lista.RemoveAt( i );
							return false;
						}
					}

					if (par.ToUpper().Equals( "-A" ))
					{
						scandirs = true;
						lista.RemoveAt( i );
						return true;
					}

					if (par.ToUpper().Equals( "-D" ))
					{
						debugmode = true;
						lista.RemoveAt( i );
						return true;
					}

					return true;
				}

			}
			return false;
		}

		static void PrintInfo()
		{
			Console.WriteLine( "Too less parameters!" );
			Console.WriteLine( "" );
			Console.WriteLine( "For file info use:" );
			Console.WriteLine( "" );
			Console.WriteLine( "\tcomparercmd.exe file" );
			Console.WriteLine( "" );
			Console.WriteLine( "where \"file\" is the name of your file." );
			Console.WriteLine( "" );
			Console.WriteLine( "To see version info use:" );
			Console.WriteLine( "" );
			Console.WriteLine( "\tcomparercmd.exe -ver" );
			Console.WriteLine( "" );
			Console.WriteLine( "Parameters:" );
			Console.WriteLine( "" );
			Console.WriteLine( "-h\t- prints out HTML info file" );						
			Console.WriteLine( "-c %C\t- uses config file specified by \"%C\" parameter" );
			Console.WriteLine( "-a\t- scans plugins dir automatically" );
			Console.WriteLine( "-d\t- shows additional info about empty plugins" );
			Console.WriteLine( "" );
			Console.WriteLine( "NOTES:" );
			Console.WriteLine( "" );
			Console.WriteLine( "If \"header.html\" file exist in ComparerCmd.exe directory, its content will be placed above table in generated HTML file" );
			Console.WriteLine( "" );
			Console.WriteLine( "If \"footer.html\" file exist in ComparerCmd.exe directory, its content will be placed below table in generated HTML file" );
		}

		static void SearchPlugins( String sciezka, ref List<String> listwdx )
		{
			//Console.WriteLine(sciezka);

			if (Directory.Exists( sciezka ))
			{

				DirectoryInfo di = new DirectoryInfo( sciezka );

				FileInfo[] fileswdx = di.GetFiles( "*.wdx" );

				foreach (FileInfo plikwdx in fileswdx)
				{
					listwdx.Add( plikwdx.FullName );
				}

				DirectoryInfo[] dirs = di.GetDirectories();

				foreach (DirectoryInfo dir in dirs)
				{
					SearchPlugins( dir.FullName, ref listwdx );
				}
			}
		}

		static Boolean WDXExist( String elem, List<Plug> lista )
		{
			for (int i = 0; i < lista.Count; i++)
			{
				if (lista[i]._path.ToUpper().Equals( elem.ToUpper() ))
				{
					return true;
				}
			}

			return false;
		}
	}
}
